import pytest
from denorm.models import DirtyInstance
from django.core.exceptions import ValidationError
from django.db import InternalError

from bpp.models import Autor_Dyscyplina


def test_autor_dyscyplina_save_ta_sama_clean(
    autor_jan_kowalski, dyscyplina1, dyscyplina2, rok
):
    """Sprawdź funkcjonowanie triggera bazodanowego przy wrzuceniu tej samej
    dyscypliny do Autor_Dyscyplina (czyli nie 'save' tylko testujemy trigger
    po stronie bazy danych)."""
    ad = Autor_Dyscyplina.objects.create(
        rok=rok,
        autor=autor_jan_kowalski,
        dyscyplina_naukowa=dyscyplina1,
        subdyscyplina_naukowa=dyscyplina2,
    )

    ad.subdyscyplina_naukowa = dyscyplina1

    with pytest.raises(ValidationError):
        ad.clean()


def test_autor_dyscyplina_save_str(autor_jan_kowalski, dyscyplina1, dyscyplina2, rok):
    ad = Autor_Dyscyplina.objects.create(
        rok=rok,
        autor=autor_jan_kowalski,
        dyscyplina_naukowa=dyscyplina1,
        subdyscyplina_naukowa=dyscyplina2,
    )
    assert "Kowalski" in str(ad)


def test_autor_dyscyplina_save_ta_sama_clean_nie_wpisano(
    autor_jan_kowalski, dyscyplina1, dyscyplina2, rok
):
    """Sprawdź funkcjonowanie funkcji 'clean' gdy nie wpisano dyscypliny"""
    ad = Autor_Dyscyplina(
        rok=rok,
        autor=autor_jan_kowalski,
        dyscyplina_naukowa=None,
        subdyscyplina_naukowa=None,
    )

    ad.clean()


def test_autor_dyscyplina_save_ta_sama_trigger(
    autor_jan_kowalski, dyscyplina1, dyscyplina2, rok
):
    """Sprawdź funkcjonowanie triggera bazodanowego przy wrzuceniu tej samej
    dyscypliny do Autor_Dyscyplina (czyli nie 'save' tylko testujemy trigger
    po stronie bazy danych)."""

    with pytest.raises(InternalError):
        Autor_Dyscyplina.objects.create(
            rok=rok,
            autor=autor_jan_kowalski,
            dyscyplina_naukowa=dyscyplina1,
            subdyscyplina_naukowa=dyscyplina1,
        )


def test_autor_dyscyplina_procent_ponad(
    autor_jan_kowalski, dyscyplina1, dyscyplina2, rok
):
    ad = Autor_Dyscyplina.objects.create(
        rok=rok,
        autor=autor_jan_kowalski,
        dyscyplina_naukowa=dyscyplina1,
        procent_dyscypliny=100,
        subdyscyplina_naukowa=dyscyplina2,
        procent_subdyscypliny=1,
    )

    with pytest.raises(ValidationError):
        ad.clean()


def test_autor_dyscyplina_dopisanie_regula_nr_1(
    autor_jan_kowalski,
    dyscyplina1,
    dyscyplina2,
    wydawnictwo_ciagle,
    rok,
    jednostka,
    typy_odpowiedzialnosci,
):
    wydawnictwo_ciagle.rok = rok
    wydawnictwo_ciagle.save()

    wca = wydawnictwo_ciagle.dodaj_autora(
        autor_jan_kowalski, jednostka, dyscyplina_naukowa=None
    )

    Autor_Dyscyplina.objects.create(
        autor=autor_jan_kowalski,
        rok=rok,
        dyscyplina_naukowa=dyscyplina1,
        subdyscyplina_naukowa=dyscyplina2,
    )

    assert wca.dyscyplina_naukowa is None
    wca.refresh_from_db()
    assert wca.dyscyplina_naukowa is None


def test_autor_dyscyplina_zmiana_dyscypliny_regula_2(
    autor_jan_kowalski,
    jednostka,
    dyscyplina1,
    dyscyplina2,
    dyscyplina3,
    rok,
    wydawnictwo_ciagle,
    wydawnictwo_zwarte,
    typy_odpowiedzialnosci,
):
    """Sprawdź, czy zmiana przypisania Autor_Dyscyplina na dany rok pociągnie zmianę w wydawnictwach
    ciągłych, zwartych i patentach. Skasowanie przypisania z kolei ustawi przypisanie na NULL."""

    Autor_Dyscyplina.objects.create(
        rok=rok + 50,
        autor=autor_jan_kowalski,
        dyscyplina_naukowa=dyscyplina1,
    )

    ad = Autor_Dyscyplina.objects.create(
        rok=rok,
        autor=autor_jan_kowalski,
        dyscyplina_naukowa=dyscyplina1,
        subdyscyplina_naukowa=dyscyplina2,
    )

    wydawnictwo_zwarte.rok = rok + 50
    wydawnictwo_zwarte.save()

    wza = wydawnictwo_zwarte.dodaj_autora(
        autor=autor_jan_kowalski,
        jednostka=jednostka,
        zapisany_jako="Tu sie nie zmieni bo bedzie inny rok",
        dyscyplina_naukowa=dyscyplina1,
    )

    wca = wydawnictwo_ciagle.dodaj_autora(
        autor=autor_jan_kowalski,
        jednostka=jednostka,
        zapisany_jako="J. Kowalski",
        dyscyplina_naukowa=dyscyplina1,
    )

    ad.dyscyplina_naukowa = dyscyplina3
    ad.save()

    wca.refresh_from_db()
    assert wca.dyscyplina_naukowa == dyscyplina3

    wza.refresh_from_db()
    assert wza.dyscyplina_naukowa == dyscyplina1

    ad.delete()

    wca.refresh_from_db()
    assert wca.dyscyplina_naukowa is None

    wza.refresh_from_db()
    assert wza.dyscyplina_naukowa == dyscyplina1


def test_autor_dyscyplina_zmiana_subdyscypliny_na_pusta_regula_3(
    autor_jan_kowalski,
    jednostka,
    dyscyplina1,
    dyscyplina2,
    rok,
    wydawnictwo_ciagle,
    typy_odpowiedzialnosci,
):
    """Sprawdź, czy zmiana przypisania Autor_Dyscyplina na dany rok pociągnie zmianę w wydawnictwach
    ciągłych. Zmieniamy subdyscypline na pusta. Wydawnictwa powiazane maja miec ustawione
    NULL"""

    ad = Autor_Dyscyplina.objects.create(
        rok=rok,
        autor=autor_jan_kowalski,
        dyscyplina_naukowa=dyscyplina1,
        subdyscyplina_naukowa=dyscyplina2,
    )

    wca = wydawnictwo_ciagle.dodaj_autora(
        autor=autor_jan_kowalski,
        jednostka=jednostka,
        zapisany_jako="J. Kowalski",
        dyscyplina_naukowa=dyscyplina2,
    )

    ad.subdyscyplina_naukowa = None
    ad.save()

    wca.refresh_from_db()
    assert wca.dyscyplina_naukowa is None


def test_autor_dyscyplina_change_trigger_subdys_from_none_bug(
    autor_jan_kowalski, dyscyplina1, dyscyplina2, rok
):
    """Sprawdź, czy zmiana przypisania Autor_Dyscyplina na dany rok pociągnie zmianę w wydawnictwach
    ciągłych, zwartych i patentach. Skasowanie przypisania z kolei ustawi przypisanie na NULL."""

    ad = Autor_Dyscyplina.objects.create(
        rok=rok + 5,
        autor=autor_jan_kowalski,
        dyscyplina_naukowa=dyscyplina1,
        subdyscyplina_naukowa=None,
    )

    ad.subdyscyplina_naukowa = dyscyplina2
    ad.save()  # Tu był bug przy zmianie z None na coś.


def test_autor_dyscyplina_zmiana_dyscypliny_regula_4(
    autor_jan_kowalski,
    jednostka,
    dyscyplina1,
    dyscyplina2,
    rok,
    wydawnictwo_ciagle,
    wydawnictwo_zwarte,
    typy_odpowiedzialnosci,
):
    """Sprawdź, czy zmiana przypisania Autor_Dyscyplina na dany rok pociągnie zmianę w wydawnictwach
    ciągłych, zwartych.

    Skasowanie przypisania na dany rok ustawi dyscypliny na NULL."""

    ad = Autor_Dyscyplina.objects.create(
        rok=rok,
        autor=autor_jan_kowalski,
        dyscyplina_naukowa=dyscyplina1,
        subdyscyplina_naukowa=dyscyplina2,
    )

    wza = wydawnictwo_zwarte.dodaj_autora(
        autor=autor_jan_kowalski, jednostka=jednostka, dyscyplina_naukowa=dyscyplina1
    )

    wca = wydawnictwo_ciagle.dodaj_autora(
        autor=autor_jan_kowalski, jednostka=jednostka, dyscyplina_naukowa=dyscyplina2
    )

    ad.delete()

    wca.refresh_from_db()
    assert wca.dyscyplina_naukowa is None

    wza.refresh_from_db()
    assert wza.dyscyplina_naukowa is None


def test_autor_dyscyplina_change_trigger_double(
    autor_jan_kowalski,
    jednostka,
    dyscyplina1,
    dyscyplina2,
    rok,
    wydawnictwo_ciagle,
    wydawnictwo_zwarte,
    patent,
    typy_odpowiedzialnosci,
):
    """Sprawdź, czy zmiana przypisania Autor_Dyscyplina na dany rok pociągnie zmianę w wydawnictwach
    ciągłych, zwartych i patentach. Skasowanie przypisania z kolei ustawi przypisanie na NULL."""

    ad = Autor_Dyscyplina.objects.create(
        rok=rok,
        autor=autor_jan_kowalski,
        dyscyplina_naukowa=dyscyplina1,
        subdyscyplina_naukowa=dyscyplina2,
    )

    wca = wydawnictwo_ciagle.dodaj_autora(
        autor=autor_jan_kowalski,
        jednostka=jednostka,
        zapisany_jako="J. Kowalski",
        dyscyplina_naukowa=dyscyplina1,
    )

    wza = wydawnictwo_zwarte.dodaj_autora(
        autor=autor_jan_kowalski,
        jednostka=jednostka,
        zapisany_jako="Kowalski Jan",
        dyscyplina_naukowa=dyscyplina2,
    )

    ad.dyscyplina_naukowa = dyscyplina2
    ad.subdyscyplina_naukowa = dyscyplina1
    ad.save()

    wca.refresh_from_db()
    assert wca.dyscyplina_naukowa == dyscyplina2

    wza.refresh_from_db()
    assert wza.dyscyplina_naukowa == dyscyplina1


def test_autor_dyscyplina_zmiana_roku(autor_jan_kowalski, dyscyplina1, rok):
    """Sprawdź funkcjonowanie triggera bazodanowego przy wrzuceniu tej samej
    dyscypliny do Autor_Dyscyplina (czyli nie 'save' tylko testujemy trigger
    po stronie bazy danych)."""

    ad = Autor_Dyscyplina.objects.create(
        rok=rok,
        autor=autor_jan_kowalski,
        dyscyplina_naukowa=dyscyplina1,
    )

    ad.rok = rok + 50
    with pytest.raises(InternalError):
        ad.save()


def test_autor_dyscyplina_zmiana_autora(
    autor_jan_kowalski, autor_jan_nowak, dyscyplina1, rok
):
    ad = Autor_Dyscyplina.objects.create(
        rok=rok,
        autor=autor_jan_kowalski,
        dyscyplina_naukowa=dyscyplina1,
    )

    ad.autor = autor_jan_nowak
    with pytest.raises(InternalError):
        ad.save()


def test_autor_dyscyplina_zmiana_z_none_na_cos(
    autor_jan_kowalski, autor_jan_nowak, dyscyplina1, dyscyplina2, rok
):
    ad = Autor_Dyscyplina.objects.create(
        rok=rok, autor=autor_jan_kowalski, dyscyplina_naukowa=dyscyplina1
    )

    ad.subdyscyplina_naukowa = dyscyplina2
    ad.save()


def test_autor_dyscyplina_cacher_zmiana(
    autor_jan_kowalski,
    jednostka,
    wydawnictwo_ciagle,
    rok,
    dyscyplina1,
    dyscyplina2,
    denorms,
):

    ad = Autor_Dyscyplina.objects.create(
        rok=rok, autor=autor_jan_kowalski, dyscyplina_naukowa=dyscyplina1
    )

    wca = wydawnictwo_ciagle.dodaj_autora(
        autor_jan_kowalski, jednostka, dyscyplina_naukowa=dyscyplina1
    )

    ad.dyscyplina_naukowa = dyscyplina2
    denorms.flush()
    ad.save()

    wca.refresh_from_db()
    assert wca.dyscyplina_naukowa == dyscyplina2
    assert DirtyInstance.objects.count() == 3


def test_autor_dyscyplina_cacher_skasowanie(
    autor_jan_kowalski,
    jednostka,
    wydawnictwo_ciagle,
    rok,
    dyscyplina1,
    dyscyplina2,
    denorms,
):
    ad = Autor_Dyscyplina.objects.create(
        rok=rok, autor=autor_jan_kowalski, dyscyplina_naukowa=dyscyplina1
    )

    wca = wydawnictwo_ciagle.dodaj_autora(
        autor_jan_kowalski, jednostka, dyscyplina_naukowa=dyscyplina1
    )

    denorms.flush()
    ad.delete()

    wca.refresh_from_db()
    assert wca.dyscyplina_naukowa is None
    assert DirtyInstance.objects.count() == 3



# Generated by Django 3.2.14 on 2022-07-07 22:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("import_dbf", "0002_auto_20200518_0056"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="b_b",
            options={"managed": False, "ordering": ("idt__tytul_or_s", "lp")},
        ),
        migrations.AlterField(
            model_name="bib_desc",
            name="value",
            field=models.JSONField(),
        ),
    ]



from unittest.mock import MagicMock, Mock

import pytest

from integrator2.models import ListaMinisterialnaIntegration


@pytest.mark.django_db
def test_models_base(normal_django_user):

    bi = ListaMinisterialnaIntegration()  # BaseIntegration()

    bi.file.name = "foo"
    assert bi.filename() == "foo"

    filter = Mock()
    bi.klass = Mock(objects=Mock(filter=filter))
    bi.records()
    assert filter.called

    bi.integrated()
    bi.not_integrated()
    assert filter.call_count == 3

    bi.match_single_record = Mock()
    elem = Mock()
    bi.records = Mock(return_value=[elem])
    bi.match_records()
    assert elem.zanalizowano
    assert elem.save.call_count == 1
    assert bi.match_single_record.called

    elem = Mock()
    bi.records = MagicMock()
    bi.records().filter.return_value = [elem]
    [x for x in bi.records().filter()]

    bi.integrate_single_record = Mock()
    bi.integrate()
    assert bi.integrate_single_record.call_count == 1
    assert elem.zintegrowano
    assert elem.save.called



import pytest
from django.urls import NoReverseMatch
from django.urls.base import reverse
from model_bakery import baker

from bpp.models import Autor, Jednostka, Praca_Doktorska, Praca_Habilitacyjna, Zrodlo
from bpp.models.cache import Autorzy, Rekord
from bpp.models.patent import Patent, Patent_Autor
from bpp.models.wydawnictwo_ciagle import Wydawnictwo_Ciagle, Wydawnictwo_Ciagle_Autor
from bpp.models.wydawnictwo_zwarte import Wydawnictwo_Zwarte, Wydawnictwo_Zwarte_Autor


@pytest.mark.parametrize(
    "klass",
    [
        Wydawnictwo_Ciagle,
        Wydawnictwo_Zwarte,
        Patent,
        Praca_Doktorska,
        Praca_Habilitacyjna,
    ],
)
def test_safe_html_dwa_tytuly_DwaTytuly(
    klass,
    admin_app,
    typy_odpowiedzialnosci,
):
    """Upewnij sie, ze bleach jest uruchamiany dla tych dwóch pól z DwaTytuly"""

    i = baker.make(klass, rok=2020)
    if hasattr(i, "zrodlo"):
        z = baker.make(Zrodlo)
        i.zrodlo = z
        i.save()

    if hasattr(i, "promotor"):
        p = baker.make(Autor)
        i.promotor = p
        i.save()

    url = reverse(f"admin:bpp_{klass._meta.model_name}_change", args=(i.pk,))

    page = admin_app.get(url)

    page.forms[1]["tytul_oryginalny"].value = "<script>hi</script>"
    if hasattr(i, "tytul"):
        page.forms[1]["tytul"].value = "<script>hi</script>"

    page.forms[1].submit()

    i.refresh_from_db()

    assert i.tytul_oryginalny == "hi"
    if hasattr(i, "tytul"):
        assert i.tytul == "hi"


@pytest.mark.parametrize(
    "klass,autor_klass,name,url",
    [
        (
            Wydawnictwo_Ciagle,
            Wydawnictwo_Ciagle_Autor,
            "wydawnictwo_ciagle",
            "admin:bpp_wydawnictwo_ciagle_change",
        ),
        (
            Wydawnictwo_Zwarte,
            Wydawnictwo_Zwarte_Autor,
            "wydawnictwo_zwarte",
            "admin:bpp_wydawnictwo_zwarte_change",
        ),
        (Patent, Patent_Autor, "patent", "admin:bpp_patent_change"),
    ],
)
def test_zapisz_wydawnictwo_w_adminie(klass, autor_klass, name, url, admin_app):

    if klass == Wydawnictwo_Ciagle:
        wc = baker.make(klass, zrodlo__nazwa="Kopara", rok=2020)
    else:
        wc = baker.make(klass, rok=2020)

    wca = baker.make(
        autor_klass,
        autor__imiona="Jan",
        autor__nazwisko="Kowalski",
        zapisany_jako="Jan Kowalski",
        rekord=wc,
    )

    url = reverse(url, args=(wc.pk,))
    res = admin_app.get(url)

    form = res.forms[name + "_form"]

    ZMIENIONE = "J[an] Kowalski"
    form["autorzy_set-0-zapisany_jako"].options.append((ZMIENIONE, False, ZMIENIONE))
    form["autorzy_set-0-zapisany_jako"].value = ZMIENIONE

    res2 = form.submit().maybe_follow()
    assert res2.status_code == 200
    assert "Please correct the error" not in res2.text
    assert "Proszę, popraw poniższe błędy." not in res2.text

    wca.refresh_from_db()
    assert wca.zapisany_jako == ZMIENIONE

    Rekord.objects.all().delete()
    Autorzy.objects.all().delete()


from django.apps import apps


@pytest.mark.parametrize("model", apps.get_models())
@pytest.mark.django_db
def test_widok_admina(admin_client, model):
    """Wejdź na podstrony admina 'changelist' oraz 'add' dla każdego modelu z aplikacji
    'bpp' który to istnieje w adminie (został zarejestrowany) i do którego to admin_client
    ma uprawnienia.

    W ten sposób możemy wyłapać błędy z nazwami pól w adminie, których to Django nie wyłapie
    przed uruchomieniem aplikacji.
    """

    # for model in apps.get_models():
    app_label = model._meta.app_label
    model_name = model._meta.model_name

    if app_label != "bpp":
        return

    url_name = f"admin:{app_label}_{model_name}_changelist"
    try:
        url = reverse(url_name)
    except NoReverseMatch:
        return

    res = admin_client.get(url)
    assert res.status_code == 200, "changelist failed for %r" % model

    res = admin_client.get(url + "?q=fafa")
    assert res.status_code == 200, "changelist query failed for %r" % model

    MODELS_WITHOUT_ADD = [("bpp", "bppmultiseekvisibility")]
    if (app_label, model_name) in MODELS_WITHOUT_ADD:
        return

    url_name = f"admin:{app_label}_{model_name}_add"
    url = reverse(url_name)
    res = admin_client.get(url)

    assert res.status_code == 200, "add failed for %r" % model


@pytest.mark.django_db
def test_admin_jednostka_sortowanie(uczelnia, admin_client):
    url_name = reverse("admin:bpp_jednostka_changelist")

    baker.make(Jednostka)
    baker.make(Jednostka)
    baker.make(Jednostka)

    uczelnia.sortuj_jednostki_alfabetycznie = True
    uczelnia.save()

    assert admin_client.get(url_name).status_code == 200

    uczelnia.sortuj_jednostki_alfabetycznie = False
    uczelnia.save()

    assert admin_client.get(url_name).status_code == 200


@pytest.mark.django_db
@pytest.mark.parametrize("url", ["wydawnictwo_zwarte", "wydawnictwo_ciagle"])
def test_admin_zewnetrzna_baza_danych(admin_client, url):
    url_name = reverse(f"admin:bpp_{url}_add")
    res = admin_client.get(url_name)
    assert "z zewn. bazami" in res.content.decode("utf-8")


@pytest.mark.django_db
def test_BppTemplateAdmin_zapis_dobrej_templatki(admin_app):
    url = reverse("admin:dbtemplates_template_add")
    res = admin_app.get(url)
    res.forms["template_form"]["content"] = "dobry content"
    res.forms["template_form"]["name"] = "nazwa.html"
    res = res.forms["template_form"].submit().maybe_follow()
    res.mustcontain("został dodany pomyślnie")


@pytest.mark.django_db
def test_BppTemplateAdmin_zapis_zlej_templatki(admin_app):
    url = reverse("admin:dbtemplates_template_add")
    res = admin_app.get(url)
    res.forms["template_form"]["content"] = "dobry content{%if koparka %}"
    res.forms["template_form"]["name"] = "nazwa.html"
    res = res.forms["template_form"].submit().maybe_follow()
    res.mustcontain("Błąd przy próbie analizy")



from django import forms
from multiseek.models import SearchForm

from ..models import (
    BppUser,
    Funkcja_Autora,
    Grupa_Pracownicza,
    Jezyk,
    Rodzaj_Zrodla,
    Status_Korekty,
    Typ_KBN,
    Typ_Odpowiedzialnosci,
    Tytul,
    Wymiar_Etatu,
    Zrodlo_Informacji,
)
from ..models.nagroda import OrganPrzyznajacyNagrody
from ..models.system import Charakter_PBN
from .autor import AutorAdmin  # noqa
from .autor_dyscyplina import Autor_DyscyplinaAdmin  # noqa
from .charakter_formalny import Charakter_FormalnyAdmin  # noqa
from .core import (
    BaseBppAdminMixin,
    PreventDeletionAdmin,
    RestrictDeletionToAdministracjaGroupAdmin,
    RestrictDeletionToAdministracjaGroupMixin,
)
from .dyscyplina_naukowa import Dyscyplina_NaukowaAdmin  # noqa
from .jednostka import JednostkaAdmin  # NOQA
from .kierunek_studiow import Kierunek_StudiowAdmin  # noqa
from .konferencja import KonferencjaAdmin  # noqa
from .patent import Patent_Admin  # noqa
from .praca_doktorska import Praca_DoktorskaAdmin  # noqa
from .praca_habilitacyjna import Praca_HabilitacyjnaAdmin  # noqa
from .seria_wydawnicza import Seria_WydawniczaAdmin  # noqa
from .szablondlaopisubibliograficznego import SzablonDlaOpisuBibliograficznego  # noqa
from .uczelnia import UczelniaAdmin  # NOQA
from .wydawca import WydawcaAdmin  # noqa
from .wydawnictwo_ciagle import Wydawnictwo_CiagleAdmin  # noqa
from .wydawnictwo_ciagle_autor import Wydawnictwo_Ciagle_Autor_Admin  # noqa
from .wydawnictwo_zwarte import (  # noqa
    Wydawnictwo_ZwarteAdmin,
    Wydawnictwo_ZwarteAdmin_Baza,
)
from .wydawnictwo_zwarte_autor import Wydawnictwo_Zwarte_Autor_Admin  # noqa
from .wydzial import WydzialAdmin  # noqa

from django.contrib import admin
from django.contrib.auth.forms import UserCreationForm

from .bppmultiseekvisibility import BppMulitiseekVisibilityAdmin  # noqa
from bpp.models import Rodzaj_Prawa_Patentowego, Zewnetrzna_Baza_Danych

# Proste tabele
from bpp.models.openaccess import (
    Czas_Udostepnienia_OpenAccess,
    Licencja_OpenAccess,
    Tryb_OpenAccess_Wydawnictwo_Ciagle,
    Tryb_OpenAccess_Wydawnictwo_Zwarte,
    Wersja_Tekstu_OpenAccess,
)


class JezykAdmin(RestrictDeletionToAdministracjaGroupAdmin):
    list_display = ["nazwa", "skrot", "pbn_uid", "skrot_crossref"]


class Funkcja_AutoraAdmin(RestrictDeletionToAdministracjaGroupAdmin):
    list_display = ["nazwa", "skrot", "pokazuj_za_nazwiskiem"]


admin.site.register(Jezyk, JezykAdmin)
admin.site.register(Funkcja_Autora, Funkcja_AutoraAdmin)
admin.site.register(Rodzaj_Zrodla, PreventDeletionAdmin)
admin.site.register(Status_Korekty, PreventDeletionAdmin)
admin.site.register(Zrodlo_Informacji, PreventDeletionAdmin)
admin.site.register(Rodzaj_Prawa_Patentowego, PreventDeletionAdmin)

admin.site.register(OrganPrzyznajacyNagrody, PreventDeletionAdmin)

admin.site.register(Grupa_Pracownicza, PreventDeletionAdmin)
admin.site.register(Wymiar_Etatu, PreventDeletionAdmin)


@admin.register(Zewnetrzna_Baza_Danych)
class Zewnetrzna_Baza_DanychAdmin(
    RestrictDeletionToAdministracjaGroupAdmin, BaseBppAdminMixin, admin.ModelAdmin
):
    list_display = ["nazwa", "skrot"]


class Charakter_PBNAdmin(
    RestrictDeletionToAdministracjaGroupMixin, BaseBppAdminMixin, admin.ModelAdmin
):
    list_display = [
        "identyfikator",
        "wlasciwy_dla",
        "opis",
        "charaktery_formalne",
        "typy_kbn",
    ]
    readonly_fields = ["identyfikator", "wlasciwy_dla", "opis", "help_text"]

    def charaktery_formalne(self, rec):
        return ", ".join(
            [f"{x.nazwa} ({x.skrot})" for x in rec.charakter_formalny_set.all()]
        )

    def typy_kbn(self, rec):
        return ", ".join([f"{x.nazwa} ({x.skrot})" for x in rec.typ_kbn_set.all()])


admin.site.register(Charakter_PBN, Charakter_PBNAdmin)


class NazwaISkrotAdmin(
    RestrictDeletionToAdministracjaGroupMixin, BaseBppAdminMixin, admin.ModelAdmin
):
    list_display = ["skrot", "nazwa"]
    search_fields = ["skrot", "nazwa"]


admin.site.register(Tytul, NazwaISkrotAdmin)


class Typ_KBNAdmin(
    RestrictDeletionToAdministracjaGroupAdmin, BaseBppAdminMixin, admin.ModelAdmin
):
    list_display = ["nazwa", "skrot", "artykul_pbn", "charakter_pbn"]


admin.site.register(Typ_KBN, Typ_KBNAdmin)


class Typ_OdpowiedzialnosciAdmin(
    RestrictDeletionToAdministracjaGroupMixin, BaseBppAdminMixin, admin.ModelAdmin
):
    list_display = ["nazwa", "skrot", "typ_ogolny"]


class Tryb_OpenAccess_Wydawnictwo_CiagleAdmin(
    RestrictDeletionToAdministracjaGroupMixin, BaseBppAdminMixin, admin.ModelAdmin
):
    list_display = ["nazwa", "skrot"]


admin.site.register(
    Tryb_OpenAccess_Wydawnictwo_Ciagle, Tryb_OpenAccess_Wydawnictwo_CiagleAdmin
)


class Tryb_OpenAccess_Wydawnictwo_ZwarteAdmin(
    RestrictDeletionToAdministracjaGroupMixin, BaseBppAdminMixin, admin.ModelAdmin
):
    list_display = ["nazwa", "skrot"]


admin.site.register(
    Tryb_OpenAccess_Wydawnictwo_Zwarte, Tryb_OpenAccess_Wydawnictwo_ZwarteAdmin
)


class Czas_Udostepnienia_OpenAccessAdmin(
    RestrictDeletionToAdministracjaGroupMixin, BaseBppAdminMixin, admin.ModelAdmin
):
    list_display = ["nazwa", "skrot"]


admin.site.register(Czas_Udostepnienia_OpenAccess, Czas_Udostepnienia_OpenAccessAdmin)


class Licencja_OpenAccessAdmin(
    RestrictDeletionToAdministracjaGroupMixin, BaseBppAdminMixin, admin.ModelAdmin
):
    list_display = ["nazwa", "skrot"]


admin.site.register(Licencja_OpenAccess, Licencja_OpenAccessAdmin)


class Wersja_Tekstu_OpenAccessAdmin(
    RestrictDeletionToAdministracjaGroupMixin, BaseBppAdminMixin, admin.ModelAdmin
):
    list_display = ["nazwa", "skrot"]


admin.site.register(Wersja_Tekstu_OpenAccess, Wersja_Tekstu_OpenAccessAdmin)

admin.site.register(Typ_Odpowiedzialnosci, Typ_OdpowiedzialnosciAdmin)

# Bibliografia


from django.contrib.auth.admin import UserAdmin


class BppUserCreationForm(UserCreationForm):
    class Meta:
        model = BppUser
        fields = "__all__"

    def clean_username(self):
        # Since User.username is unique, this check is redundant,
        # but it sets a nicer error message than the ORM. See #13147.
        username = self.cleaned_data["username"]
        try:
            BppUser._default_manager.get(username=username)
        except BppUser.DoesNotExist:
            return username
        raise forms.ValidationError(
            self.error_messages["duplicate_username"],
            code="duplicate_username",
        )


class BppUserAdmin(UserAdmin):
    list_display = (
        "username",
        "email",
        "first_name",
        "last_name",
        "is_staff",
        "is_active",
        "is_superuser",
        "lista_grup",
    )

    add_form = BppUserCreationForm

    # change_form_template = 'loginas/change_form.html'

    def has_delete_permission(self, request, obj=None):
        if obj is not None:
            # Nie pozwól na usunięcie obecnego konta
            if obj == request.user:
                return False

            # Sprawdź przy kasowaniu konta 'admin' czy są jeszcze jakieś konta z
            # uprawnieniem admina, jeżeli nie, to nie pozwól na to:
            if not BppUser.objects.exclude(pk=obj.pk).exists():
                return False

        return super().has_delete_permission(request, obj)

    def lista_grup(self, row):
        return ", ".join([x.name for x in row.groups.all()])


admin.site.register(BppUser, BppUserAdmin)


class SearchFormAdmin(admin.ModelAdmin):
    list_display = ["name", "owner", "public"]
    fields = ["name", "owner", "public", "data"]
    readonly_fields = ["data"]


SearchForm._meta.verbose_name = "formularz wyszukiwania"
SearchForm._meta.verbose_name_plural = "formularze wyszukiwania"

admin.site.register(SearchForm, SearchFormAdmin)

from .templates import TemplateAdmin  # noqa



from pbn_api.admin.base import BaseMongoDBAdmin
from pbn_api.models import Conference

from django.contrib import admin


@admin.register(Conference)
class ConferenceAdmin(BaseMongoDBAdmin):
    list_display = [
        "fullName",
        "startDate",
        "endDate",
        "city",
        "country",
        "website",
        "mongoId",
    ]



from dal import autocomplete
from django import forms
from djangoql.admin import DjangoQLSearchMixin
from mptt.forms import TreeNodeChoiceField
from taggit.forms import TextareaTagWidget

from dynamic_columns.mixins import DynamicColumnsMixin
from pbn_api.models import Publication
from .actions import (
    ustaw_po_korekcie,
    ustaw_przed_korekta,
    ustaw_w_trakcie_korekty,
    wyslij_do_pbn,
)
from .core import BaseBppAdminMixin, KolumnyZeSkrotamiMixin, generuj_inline_dla_autorow
from .element_repozytorium import Element_RepozytoriumInline
from .grant import Grant_RekorduInline
from .helpers import OptionalPBNSaveMixin, sprawdz_duplikaty_www_doi
from .nagroda import NagrodaInline
from .util import CustomizableFormsetParamsAdminMixinWyrzucWDjango40

# Proste tabele
from .wydawnictwo_ciagle import CleanDOIWWWPublicWWWMixin
from .xlsx_export import resources
from .xlsx_export.mixins import EksportDanychMixin
from .zglos_publikacje_helpers import UzupelniajWstepneDanePoNumerzeZgloszeniaMixin

from django.contrib import admin, messages

from bpp.admin import helpers
from bpp.admin.filters import (
    CalkowitaLiczbaAutorowFilter,
    DOIUstawioneFilter,
    JestWydawnictwemNadrzednymDlaFilter,
    LiczbaZnakowFilter,
    MaWydawnictwoNadrzedneFilter,
    OstatnioZmienionePrzezFilter,
    PBN_UID_IDObecnyFilter,
    UtworzonePrzezFilter,
)
from bpp.models import (
    Charakter_Formalny,
    Wydawca,
    Wydawnictwo_Zwarte,
    Wydawnictwo_Zwarte_Autor,
    Wydawnictwo_Zwarte_Streszczenie,
    Wydawnictwo_Zwarte_Zewnetrzna_Baza_Danych,
)
from bpp.models.konferencja import Konferencja
from bpp.models.seria_wydawnicza import Seria_Wydawnicza


class Wydawnictwo_Zwarte_StreszczenieInline(admin.StackedInline):
    model = Wydawnictwo_Zwarte_Streszczenie
    extra = 0
    fields = ["jezyk_streszczenia", "streszczenie"]


class Wydawnictwo_ZwarteAdmin_Baza(BaseBppAdminMixin, admin.ModelAdmin):
    formfield_overrides = helpers.NIZSZE_TEXTFIELD_Z_MAPA_ZNAKOW

    actions = [
        ustaw_po_korekcie,
        ustaw_w_trakcie_korekty,
        ustaw_przed_korekta,
        wyslij_do_pbn,
    ]

    list_display_always = [
        "tytul_oryginalny",
    ]

    list_display_default = [
        "wydawnictwo",
        "doi",
        "wydawnictwo_nadrzedne_col",
        "rok",
        "typ_kbn__skrot",
        "charakter_formalny__skrot",
        "ostatnio_zmieniony",
    ]

    list_display_allowed = "__all__"

    search_fields = [
        "tytul",
        "tytul_oryginalny",
        "szczegoly",
        "uwagi",
        "informacje",
        "slowa_kluczowe__name",
        "rok",
        "isbn",
        "id",
        "wydawca__nazwa",
        "wydawca_opis",
        "redakcja",
        "adnotacje",
        "liczba_znakow_wydawniczych",
        "wydawnictwo_nadrzedne__tytul_oryginalny",
        "konferencja__nazwa",
        "liczba_znakow_wydawniczych",
        "doi",
        "pbn_uid__pk",
    ]

    list_filter = [
        "status_korekty",
        "recenzowana",
        "typ_kbn",
        "charakter_formalny",
        "informacja_z",
        "jezyk",
        LiczbaZnakowFilter,
        "rok",
        JestWydawnictwemNadrzednymDlaFilter,
        MaWydawnictwoNadrzedneFilter,
        DOIUstawioneFilter,
        OstatnioZmienionePrzezFilter,
        UtworzonePrzezFilter,
        PBN_UID_IDObecnyFilter,
    ]

    # Usunąć przed wcomitowaniem

    # fieldsets = (
    #     ('Wydawnictwo zwarte', {
    #         'fields':
    #             DWA_TYTULY
    #             + MODEL_ZE_SZCZEGOLAMI
    #             + ('miejsce_i_rok', 'wydawnictwo',)
    #             + MODEL_Z_ISBN
    #             + MODEL_Z_ROKIEM
    #     }),
    #     EKSTRA_INFORMACJE_WYDAWNICTWO_ZWARTE_FIELDSET,
    #     MODEL_TYPOWANY_FIELDSET,
    #     MODEL_PUNKTOWANY_FIELDSET,
    #     MODEL_PUNKTOWANY_KOMISJA_CENTRALNA_FIELDSET,
    #     POZOSTALE_MODELE_FIELDSET,
    #     ADNOTACJE_Z_DATAMI_ORAZ_PBN_FIELDSET)

    def wydawnictwo_nadrzedne_col(self, obj):
        try:
            return obj.wydawnictwo_nadrzedne.tytul_oryginalny
        except Wydawnictwo_Zwarte.DoesNotExist:
            return ""
        except AttributeError:
            return ""

    wydawnictwo_nadrzedne_col.short_description = "Wydawnictwo nadrzędne"
    wydawnictwo_nadrzedne_col.admin_order_field = (
        "wydawnictwo_nadrzedne__tytul_oryginalny"
    )


class Wydawnictwo_ZwarteForm(
    helpers.Wycinaj_W_z_InformacjiMixin, CleanDOIWWWPublicWWWMixin, forms.ModelForm
):
    wydawnictwo_nadrzedne = forms.ModelChoiceField(
        required=False,
        queryset=Wydawnictwo_Zwarte.objects.all(),
        widget=autocomplete.ModelSelect2(
            url="bpp:wydawnictwo-nadrzedne-autocomplete",
            attrs=dict(style="width: 746px;"),
        ),
    )

    wydawca = forms.ModelChoiceField(
        required=False,
        queryset=Wydawca.objects.all(),
        widget=autocomplete.ModelSelect2(
            url="bpp:wydawca-autocomplete", attrs=dict(style="width: 746px;")
        ),
    )

    konferencja = forms.ModelChoiceField(
        required=False,
        queryset=Konferencja.objects.all(),
        widget=autocomplete.ModelSelect2(
            url="bpp:konferencja-autocomplete", attrs=dict(style="width: 746px;")
        ),
    )

    pbn_uid = forms.ModelChoiceField(
        label="Odpowiednik w PBN",
        required=False,
        queryset=Publication.objects.all(),
        widget=autocomplete.ModelSelect2(
            url="bpp:publication-autocomplete", attrs=dict(style="width: 746px;")
        ),
    )

    seria_wydawnicza = forms.ModelChoiceField(
        required=False,
        queryset=Seria_Wydawnicza.objects.all(),
        widget=autocomplete.ModelSelect2(
            url="bpp:seria-wydawnicza-autocomplete",
            attrs=dict(style="width: 746px;"),
        ),
    )

    charakter_formalny = TreeNodeChoiceField(
        required=True, queryset=Charakter_Formalny.objects.all()
    )

    status_korekty = helpers.DomyslnyStatusKorektyMixin.status_korekty

    class Meta:
        fields = "__all__"
        widgets = {
            "strony": forms.TextInput(attrs=dict(style="width: 150px")),
            "tom": forms.TextInput(attrs=dict(style="width: 150px")),
            "slowa_kluczowe": TextareaTagWidget(attrs={"rows": 2}),
        }


class Wydawnictwo_Zwarte_Zewnetrzna_Baza_DanychForm(forms.ModelForm):
    class Meta:
        fields = ["baza", "info"]


class Wydawnictwo_Zwarte_Zewnetrzna_Baza_DanychInline(admin.StackedInline):
    model = Wydawnictwo_Zwarte_Zewnetrzna_Baza_Danych
    extra = 0
    form = Wydawnictwo_Zwarte_Zewnetrzna_Baza_DanychForm


class Wydawnictwo_ZwarteAdmin(
    DjangoQLSearchMixin,
    KolumnyZeSkrotamiMixin,
    helpers.AdnotacjeZDatamiOrazPBNMixin,
    OptionalPBNSaveMixin,
    EksportDanychMixin,
    UzupelniajWstepneDanePoNumerzeZgloszeniaMixin,
    CustomizableFormsetParamsAdminMixinWyrzucWDjango40,
    DynamicColumnsMixin,
    Wydawnictwo_ZwarteAdmin_Baza,
):
    form = Wydawnictwo_ZwarteForm
    djangoql_completion_enabled_by_default = False
    djangoql_completion = True
    search_fields = Wydawnictwo_ZwarteAdmin_Baza.search_fields
    resource_class = resources.Wydawnictwo_ZwarteResource

    inlines = (
        generuj_inline_dla_autorow(Wydawnictwo_Zwarte_Autor),
        NagrodaInline,
        Wydawnictwo_Zwarte_Zewnetrzna_Baza_DanychInline,
        Grant_RekorduInline,
        Element_RepozytoriumInline,
        Wydawnictwo_Zwarte_StreszczenieInline,
    )

    list_filter = Wydawnictwo_ZwarteAdmin_Baza.list_filter + [
        CalkowitaLiczbaAutorowFilter,
        "openaccess_tryb_dostepu",
        "openaccess_wersja_tekstu",
        "openaccess_licencja",
        "openaccess_czas_publikacji",
    ]

    list_select_related = {
        "__always__": ["typ_kbn", "charakter_formalny"],
        "wydawnictwo_nadrzedne": ["wydawnictwo_nadrzedne"],
        "wydawnictwo_nadrzedne_col": ["wydawnictwo_nadrzedne"],
        "wydawca": ["wydawca"],
    }

    autocomplete_fields = [
        "pbn_uid",
    ]

    fieldsets = (
        (
            "Wydawnictwo zwarte",
            {
                "fields": helpers.DWA_TYTULY
                + helpers.MODEL_ZE_SZCZEGOLAMI
                + (
                    "wydawnictwo_nadrzedne",
                    "konferencja",
                    "calkowita_liczba_autorow",
                    "calkowita_liczba_redaktorow",
                    "oznaczenie_wydania",
                    "miejsce_i_rok",
                    "wydawca",
                    "wydawca_opis",
                )
                + helpers.MODEL_Z_ISBN
                + helpers.MODEL_Z_ROKIEM
            },
        ),
        helpers.SERIA_WYDAWNICZA_FIELDSET,
        helpers.EKSTRA_INFORMACJE_WYDAWNICTWO_ZWARTE_FIELDSET,
        helpers.MODEL_TYPOWANY_FIELDSET,
        helpers.MODEL_PUNKTOWANY_FIELDSET,
        helpers.MODEL_PUNKTOWANY_KOMISJA_CENTRALNA_FIELDSET,
        helpers.POZOSTALE_MODELE_WYDAWNICTWO_ZWARTE_FIELDSET,
        helpers.ADNOTACJE_Z_DATAMI_ORAZ_PBN_FIELDSET,
        helpers.MODEL_OPCJONALNIE_NIE_EKSPORTOWANY_DO_API_FIELDSET,
        helpers.OPENACCESS_FIELDSET,
        helpers.PRACA_WYBITNA_FIELDSET,
        helpers.PRZED_PO_LISCIE_AUTOROW_FIELDSET,
        helpers.MODEL_Z_OPLATA_ZA_PUBLIKACJE_FIELDSET,
    )

    def save_model(self, request, obj, form, change):
        super().save_model(request, obj, form, change)
        if (
            obj.rok >= 2017
            and obj.rok <= 2020
            and obj.charakter_formalny.charakter_sloty is None
        ):
            messages.warning(
                request,
                'Punkty dla dyscyplin dla "%s" nie będą liczone, gdyż jest to ani książka, ani rozdział'
                % helpers.link_do_obiektu(obj),
            )
        else:
            helpers.sprobuj_policzyc_sloty(request, obj)

        sprawdz_duplikaty_www_doi(request, obj)


admin.site.register(Wydawnictwo_Zwarte, Wydawnictwo_ZwarteAdmin)



from django.contrib.messages import constants
from django.utils.functional import cached_property

from notifications import core as notifications_core
from notifications.models import Notification


class NullNotificationMixin:
    def send_notification(self, msg, level=None):
        return

    def send_processing_finished(self):
        return


class ASGINotificationMixin:
    @cached_property
    def asgi_channel_name(self):
        return str(self.pk)

    def send_notification(self, msg, level=constants.INFO):
        notifications_core.send_notification(self.asgi_channel_name, level, msg)

    def send_progress(self, percent):
        notifications_core._send(
            self.asgi_channel_name, dict(progress=True, percent=str(percent) + "%")
        )

    def send_processing_finished(self):
        # Zamiast wysyłać redirect, który może nie zostać odebrany przez klienta
        # (np. przetwarzanie skończyło się nim strona WWW została wyświetlona)
        # utwórzmy prawdziwy obiekt notyfikacji, którego odbiór wymaga potwierdzenia
        # przez zdalną przeglądarkę.
        for elem in [
            Notification.objects.create(
                channel_name=self.asgi_channel_name,
                values=dict(progress=True, percent="100%"),
            ),
            Notification.objects.create(
                channel_name=self.asgi_channel_name, values=dict(url="..")
            ),
        ]:
            elem.send()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-15 20:23
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0124_auto_20180414_1917'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='autor_dyscyplina',
            options={'verbose_name': 'powiązanie autora z dyscypliną naukową', 'verbose_name_plural': 'powiązania autorów z dyscyplinami naukowymi'},
        ),
        migrations.AlterModelOptions(
            name='dyscyplina_naukowa',
            options={'verbose_name': 'dyscyplina naukowa', 'verbose_name_plural': 'dyscypliny naukowe'},
        ),
    ]



import base64

import pytest
from django.urls import reverse
from model_bakery import baker

from bpp.models import Autor, Rekord, Wydawnictwo_Ciagle

from django_bpp.selenium_util import wait_for, wait_for_page_load


@pytest.fixture
def autor_m():
    return baker.make(
        Autor,
        # base64, bo RODO:
        nazwisko=base64.decodebytes(b"TWllbG5pY3plaw==\n").decode("ascii"),
        imiona="Katarzyna",
    )


@pytest.mark.vcr
def test_crossref_api_autor_wo_selenium(admin_app, autor_m):

    url = "/admin/bpp/wydawnictwo_ciagle/pobierz-z-crossref/"
    page = admin_app.get(url)
    page.forms[1]["identyfikator_doi"] = "10.12775/jehs.2022.12.07.045"
    page = page.forms[1].submit().maybe_follow()
    if b"id_ustaw_orcid_button_author.0" not in page.content:
        page.showbrowser()
        raise Exception


@pytest.mark.vcr(ignore_localhost=True)
@pytest.mark.flaky(reruns=1)
def test_crossref_api_autor_sync(admin_browser, live_server, transactional_db, autor_m):

    with wait_for_page_load(admin_browser):
        admin_browser.visit(
            live_server.url
            + reverse("admin:bpp_wydawnictwo_ciagle_add")
            + "../pobierz-z-crossref/"
        )

    admin_browser.find_by_name("identyfikator_doi").type("10.12775/jehs.2022.12.07.045")

    try:
        with wait_for_page_load(admin_browser):
            admin_browser.find_by_id("id_submit").click()

        admin_browser.find_by_id("id_ustaw_orcid_button_author.0")[0].click()

        def _():
            autor_m.refresh_from_db()
            return autor_m.orcid == "0000-0003-2575-3642"

        wait_for(_)
        assert True
    finally:
        autor_m.delete()


@pytest.fixture
def wydawnictwo_ciagle_jehs_2022():
    return baker.make(
        Wydawnictwo_Ciagle,
        doi="10.12775/jehs.2022.12.07.045",
        tytul_oryginalny="Neurological and neuropsychological post-covid complications",
    )


@pytest.fixture
def admin_browser_strona_porownania(admin_browser, live_server):
    with wait_for_page_load(admin_browser):
        admin_browser.visit(
            live_server.url
            + reverse("admin:bpp_wydawnictwo_ciagle_add")
            + "../pobierz-z-crossref/"
        )

    admin_browser.find_by_name("identyfikator_doi").type("10.12775/jehs.2022.12.07.045")

    with wait_for_page_load(admin_browser):
        admin_browser.find_by_id("id_submit").click()

    return admin_browser


@pytest.mark.django_db
@pytest.mark.vcr(ignore_localhost=True)
def test_crossref_api_strony_view(
    wydawnictwo_ciagle_jehs_2022,
    csrf_exempt_django_admin_app,
):

    csrf_exempt_django_admin_app.post(
        reverse("bpp:api_ustaw_strony"),
        {"rekord": Rekord.objects.all().first().form_post_pk, "strony": "447-452"},
    )

    wydawnictwo_ciagle_jehs_2022.refresh_from_db()

    return wydawnictwo_ciagle_jehs_2022.strony == "447-452"


@pytest.mark.vcr(ignore_localhost=True)
@pytest.mark.parametrize(
    "id_przycisku, atrybut, wynik",
    [
        ("id_ustaw_strony_button", "strony", "447-452"),
        ("id_ustaw_tom_button", "tom", "12"),
        ("id_ustaw_nr_zeszytu_button", "nr_zeszytu", "7"),
    ],
)
def test_crossref_api_strony_sync_browser(
    transactional_db,
    wydawnictwo_ciagle_jehs_2022,
    live_server,
    admin_browser_strona_porownania,
    id_przycisku,
    atrybut,
    wynik,
):
    # Kliknij id_przycisku, sprawdz czy atrybut wydawnictwa jest rowny do wynik
    admin_browser_strona_porownania.find_by_id(id_przycisku)[0].click()

    def _():
        wydawnictwo_ciagle_jehs_2022.refresh_from_db()
        return getattr(wydawnictwo_ciagle_jehs_2022, atrybut, None) == wynik

    wait_for(_)

    assert True


def test_crossref_api_streszczenie_sync_browser(
    transactional_db,
    wydawnictwo_ciagle_jehs_2022,
    live_server,
    admin_browser_strona_porownania,
):
    # Kliknij id_przycisku, sprawdz czy atrybut wydawnictwa jest rowny do wynik
    admin_browser_strona_porownania.find_by_id("id_ustaw_streszczenie_button").click()

    def _():
        wydawnictwo_ciagle_jehs_2022.refresh_from_db()
        return wydawnictwo_ciagle_jehs_2022.streszczenia.exists()

    wait_for(_)

    assert True



# -*- encoding: utf-8 -*-

from django.core.management import BaseCommand
from django.db import transaction
from django.db.models import Q
import psycopg2
import sys
from bpp.models import Rekord


class Command(BaseCommand):
    help = 'Weryfikuje prace bez autorow'

    @transaction.atomic
    def handle(self, *args, **options):
        connection = psycopg2.connect(database="b_med", host="linux-dev")
        cursor = connection.cursor()

        raise NotImplementedError("wybierz wszystkie rekordy BEZ autorow.")
        for praca in Rekord.objects.filter(Q(autorzy=[]) | Q(autorzy=None)):
            orig = praca.get_original_object()
            if getattr(orig, 'autorzy'):
                assert(orig.autorzy.count() == 0)
            else:
                assert(orig.autor == None)

            qry = "SELECT * FROM b_a WHERE idt = %s" % praca.object_pk
            cursor.execute(qry)
            res = cursor.fetchall()
            assert(len(res) == 0)
        print("%s: DB is correct!" % sys.argv[0])


# -*- encoding: utf-8 -*-

from dal import autocomplete
from django import forms
from django.contrib.contenttypes.admin import GenericTabularInline, GenericStackedInline

from bpp.models.nagroda import Nagroda, OrganPrzyznajacyNagrody


class NagrodaForm(forms.ModelForm):
    organ_przyznajacy = forms.ModelChoiceField(
        queryset=OrganPrzyznajacyNagrody.objects.all(),
        widget=autocomplete.ModelSelect2(
            url='bpp:organ-przyznajacy-nagrody-autocomplete')
    )

    class Meta:
        fields = ["nazwa",
                  "organ_przyznajacy",
                  "rok_przyznania",
                  "uzasadnienie",
                  "adnotacja"]
        model = Nagroda


class NagrodaInline(GenericStackedInline):
    model = Nagroda
    extra = 0
    form = NagrodaForm



from adminsortable2.admin import SortableAdminMixin

from ..models import Wydzial  # Publikacja_Habilitacyjna
from .core import BaseBppAdminMixin, RestrictDeletionToAdministracjaGroupMixin
from .helpers import ADNOTACJE_FIELDSET, ZapiszZAdnotacjaMixin

from django.contrib import admin


class WydzialAdmin(
    RestrictDeletionToAdministracjaGroupMixin,
    SortableAdminMixin,
    ZapiszZAdnotacjaMixin,
    BaseBppAdminMixin,
    admin.ModelAdmin,
):
    list_display = [
        "nazwa",
        "skrot",
        "kolejnosc",
        "widoczny",
        "ranking_autorow",
        "zarzadzaj_automatycznie",
        "otwarcie",
        "zamkniecie",
        "pbn_id",
    ]
    list_filter = [
        "uczelnia",
        "zezwalaj_na_ranking_autorow",
        "widoczny",
        "zarzadzaj_automatycznie",
    ]
    fieldsets = (
        (
            None,
            {
                "fields": (
                    "uczelnia",
                    "nazwa",
                    "skrot_nazwy",
                    "skrot",
                    "pbn_id",
                    "opis",
                    "widoczny",
                    "zezwalaj_na_ranking_autorow",
                    "zarzadzaj_automatycznie",
                    "otwarcie",
                    "zamkniecie",
                ),
            },
        ),
        ADNOTACJE_FIELDSET,
    )

    def ranking_autorow(self, obj):
        return obj.zezwalaj_na_ranking_autorow

    ranking_autorow.short_description = "Ranking autorów"
    ranking_autorow.boolean = True
    ranking_autorow.admin_order_field = "zezwalaj_na_ranking_autorow"


admin.site.register(Wydzial, WydzialAdmin)



from django.db.models import BLANK_CHOICE_DASH


class ReadOnlyListChangeFormAdminMixin:
    actions = None

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, *args, **kw):
        return False

    def has_change_permission(self, *args, **kw):
        return False

    def get_action_choices(self, request, default_choices=BLANK_CHOICE_DASH):
        return []



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-06-24 14:20
from __future__ import unicode_literals

import bpp.models.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0139_uczelnia_pokazuj_liczbe_cytowan_na_stronie_autora'),
    ]

    operations = [
        migrations.AlterField(
            model_name='konferencja',
            name='baza_inna',
            field=models.CharField(blank=True, help_text='Wpisz listę innych baz czasopism i abstraktów, w których indeksowana była ta konferencja. Rozdziel średnikiem.', max_length=200, null=True, verbose_name='Indeksowana w...'),
        ),
        migrations.AlterField(
            model_name='uczelnia',
            name='pokazuj_raport_jednostek',
            field=bpp.models.fields.OpcjaWyswietlaniaField(choices=[('always', 'zawsze'), ('logged-in', 'tylko dla zalogowanych'), ('never', 'nigdy')], default='logged-in', max_length=50, verbose_name='Pokazuj raport jednostek'),
        ),
        migrations.AlterField(
            model_name='uczelnia',
            name='pokazuj_raport_wydzialow',
            field=bpp.models.fields.OpcjaWyswietlaniaField(choices=[('always', 'zawsze'), ('logged-in', 'tylko dla zalogowanych'), ('never', 'nigdy')], default='logged-in', max_length=50, verbose_name='Pokazuj raport wydziałów'),
        ),
    ]



from django.db import models

from django.contrib.contenttypes.fields import GenericForeignKey


class IgnorujRozbieznoscIf(models.Model):
    object = GenericForeignKey()

    content_type = models.ForeignKey(
        "contenttypes.ContentType", on_delete=models.CASCADE
    )
    object_id = models.PositiveIntegerField("Rekord", db_index=True)
    created_on = models.DateTimeField("Utworzono", auto_now_add=True)

    class Meta:
        verbose_name = "ignorowanie rozbieżności impact factor"
        verbose_name = "ignorowanie rozbieżności impact factor"

    def __str__(self):
        try:
            return f"Ignoruj rozbieżności punktacji IF dla rekordu {self.object}"
        except BaseException:
            return 'Ignoruj rozbieżności punktacji IF dla rekordu "[brak rekordu, został usunięty]"'



from django.core.exceptions import ObjectDoesNotExist


def object_or_something(model, attrname, default_pk=-1, default_attr="nazwa", default_value="--", **kwargs):
    """
    Funkcja do rozsądnego zwracania czegoś w przypadku, gdy obiekt powiązany
    (ForeignKey) może być None/NULL. W takich sytuacjach Django zwraca błąd.
    Załóżmy, że chcemy coś wysłać JSONem do klienta i nie interesuje nas wartość
    NULL, ale coś rozsądnego np z etykietą.

    Funkcja zwróci albo obiekt powiązany dla modelu 'model', określony przez
    atrybut 'attrname'... albo klasę stworzoną ad-hoc, z atrybutami określonymi
    przez paramtery.

    """

    try:
        res = getattr(model, attrname)
    except ObjectDoesNotExist:
        pass

    if res is not None:
        return res

    class Unexistent:
        pk = default_pk

    ret = Unexistent()
    if not kwargs and default_attr is not None:
        kwargs[default_attr] = default_value

    for key, value in kwargs.items():
        setattr(ret, key, value)

    return ret



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src',
 'bpp': 'src/bpp',
 'bpp.admin': 'src/bpp/admin',
 'bpp.admin.xlsx_export': 'src/bpp/admin/xlsx_export',
 'bpp.context_processors': 'src/bpp/context_processors',
 'bpp.fixtures': 'src/bpp/fixtures',
 'bpp.forms': 'src/bpp/forms',
 'bpp.imports': 'src/bpp/imports',
 'bpp.imports.uml': 'src/bpp/imports/uml',
 'bpp.management': 'src/bpp/management',
 'bpp.management.commands': 'src/bpp/management/commands',
 'bpp.migrations': 'src/bpp/migrations',
 'bpp.models': 'src/bpp/models',
 'bpp.models.sloty': 'src/bpp/models/sloty',
 'bpp.multiseek_registry': 'src/bpp/multiseek_registry',
 'bpp.reports': 'src/bpp/reports',
 'bpp.templatetags': 'src/bpp/templatetags',
 'bpp.tests': 'src/bpp/tests',
 'bpp.tests.test_admin': 'src/bpp/tests/test_admin',
 'bpp.tests.test_api': 'src/bpp/tests/test_api',
 'bpp.tests.test_models': 'src/bpp/tests/test_models',
 'bpp.tests.test_models.test_abstract': 'src/bpp/tests/test_models/test_abstract',
 'bpp.tests.test_models.test_sloty': 'src/bpp/tests/test_models/test_sloty',
 'bpp.tests.test_models.test_struktura': 'src/bpp/tests/test_models/test_struktura',
 'bpp.tests.test_multiseek': 'src/bpp/tests/test_multiseek',
 'bpp.tests.test_selenium': 'src/bpp/tests/test_selenium',
 'bpp.tests.test_selenium.test_admin': 'src/bpp/tests/test_selenium/test_admin',
 'bpp.tests.test_views': 'src/bpp/tests/test_views',
 'bpp.tests.test_views.test_browse': 'src/bpp/tests/test_views/test_browse',
 'bpp.tests.tests_legacy': 'src/bpp/tests/tests_legacy',
 'bpp.tests.tests_legacy.test_reports': 'src/bpp/tests/tests_legacy/test_reports',
 'bpp.tests.tests_legacy.test_views': 'src/bpp/tests/tests_legacy/test_views',
 'bpp.views': 'src/bpp/views',
 'bpp.views.api': 'src/bpp/views/api',
 'bpp.views.autocomplete': 'src/bpp/views/autocomplete',
 'bpp.views.raporty': 'src/bpp/views/raporty',
 'celeryui': 'src/celeryui',
 'celeryui.migrations': 'src/celeryui/migrations',
 'create_test_db': 'src/create_test_db',
 'create_test_db.management': 'src/create_test_db/management',
 'create_test_db.management.commands': 'src/create_test_db/management/commands',
 'create_test_db.migrations': 'src/create_test_db/migrations',
 'crossref_bpp': 'src/crossref_bpp',
 'crossref_bpp.migrations': 'src/crossref_bpp/migrations',
 'crossref_bpp.tests': 'src/crossref_bpp/tests',
 'django_bpp': 'src/django_bpp',
 'django_bpp.settings': 'src/django_bpp/settings',
 'django_bpp.staticroot': 'src/django_bpp/staticroot',
 'django_bpp.staticroot.notifications': 'src/django_bpp/staticroot/notifications',
 'django_bpp.staticroot.notifications.js': 'src/django_bpp/staticroot/notifications/js',
 'django_bpp.staticroot.notifications.js.tests': 'src/django_bpp/staticroot/notifications/js/tests',
 'dynamic_columns': 'src/dynamic_columns',
 'dynamic_columns.migrations': 'src/dynamic_columns/migrations',
 'ewaluacja2021': 'src/ewaluacja2021',
 'ewaluacja2021.core': 'src/ewaluacja2021/core',
 'ewaluacja2021.management': 'src/ewaluacja2021/management',
 'ewaluacja2021.management.commands': 'src/ewaluacja2021/management/commands',
 'ewaluacja2021.migrations': 'src/ewaluacja2021/migrations',
 'ewaluacja2021.tests': 'src/ewaluacja2021/tests',
 'ewaluacja2021.tests.test_core': 'src/ewaluacja2021/tests/test_core',
 'fixtures': 'src/fixtures',
 'formdefaults': 'src/formdefaults',
 'formdefaults.migrations': 'src/formdefaults/migrations',
 'formdefaults.tests': 'src/formdefaults/tests',
 'import_common': 'src/import_common',
 'import_common.tests': 'src/import_common/tests',
 'import_dbf': 'src/import_dbf',
 'import_dbf.management': 'src/import_dbf/management',
 'import_dbf.management.commands': 'src/import_dbf/management/commands',
 'import_dbf.migrations': 'src/import_dbf/migrations',
 'import_dbf.tests': 'src/import_dbf/tests',
 'import_dyscyplin': 'src/import_dyscyplin',
 'import_dyscyplin.management.commands': 'src/import_dyscyplin/management/commands',
 'import_dyscyplin.migrations': 'src/import_dyscyplin/migrations',
 'import_dyscyplin.tests': 'src/import_dyscyplin/tests',
 'import_list_if': 'src/import_list_if',
 'import_list_if.migrations': 'src/import_list_if/migrations',
 'import_list_if.tests': 'src/import_list_if/tests',
 'import_pracownikow': 'src/import_pracownikow',
 'import_pracownikow.migrations': 'src/import_pracownikow/migrations',
 'import_pracownikow.tests': 'src/import_pracownikow/tests',
 'import_pracownikow.tests.test_models': 'src/import_pracownikow/tests/test_models',
 'integrator2': 'src/integrator2',
 'integrator2.migrations': 'src/integrator2/migrations',
 'integrator2.models': 'src/integrator2/models',
 'integrator2.tests': 'src/integrator2/tests',
 'long_running': 'src/long_running',
 'long_running.migrations': 'src/long_running/migrations',
 'long_running.tests': 'src/long_running/tests',
 'miniblog': 'src/miniblog',
 'miniblog.migrations': 'src/miniblog/migrations',
 'notifications': 'src/notifications',
 'notifications.management': 'src/notifications/management',
 'notifications.management.commands': 'src/notifications/management/commands',
 'notifications.migrations': 'src/notifications/migrations',
 'notifications.static': 'src/notifications/static',
 'notifications.static.notifications': 'src/notifications/static/notifications',
 'notifications.static.notifications.js': 'src/notifications/static/notifications/js',
 'notifications.static.notifications.js.tests': 'src/notifications/static/notifications/js/tests',
 'notifications.templatetags': 'src/notifications/templatetags',
 'notifications.tests': 'src/notifications/tests',
 'nowe_raporty': 'src/nowe_raporty',
 'nowe_raporty.tests': 'src/nowe_raporty/tests',
 'pbn_api': 'src/pbn_api',
 'pbn_api.adapters': 'src/pbn_api/adapters',
 'pbn_api.admin': 'src/pbn_api/admin',
 'pbn_api.conf': 'src/pbn_api/conf',
 'pbn_api.integrator': 'src/pbn_api/integrator',
 'pbn_api.management': 'src/pbn_api/management',
 'pbn_api.management.commands': 'src/pbn_api/management/commands',
 'pbn_api.migrations': 'src/pbn_api/migrations',
 'pbn_api.models': 'src/pbn_api/models',
 'pbn_api.tests': 'src/pbn_api/tests',
 'raport_slotow': 'src/raport_slotow',
 'raport_slotow.migrations': 'src/raport_slotow/migrations',
 'raport_slotow.models': 'src/raport_slotow/models',
 'raport_slotow.tests': 'src/raport_slotow/tests',
 'raport_slotow.tests.test_raport_slotow_autor': 'src/raport_slotow/tests/test_raport_slotow_autor',
 'raport_slotow.tests.test_views': 'src/raport_slotow/tests/test_views',
 'raport_slotow.tests.tests_models': 'src/raport_slotow/tests/tests_models',
 'raport_slotow.views': 'src/raport_slotow/views',
 'rozbieznosci_dyscyplin': 'src/rozbieznosci_dyscyplin',
 'rozbieznosci_dyscyplin.migrations': 'src/rozbieznosci_dyscyplin/migrations',
 'rozbieznosci_if': 'src/rozbieznosci_if',
 'rozbieznosci_if.migrations': 'src/rozbieznosci_if/migrations',
 'tee': 'src/tee',
 'tee.management': 'src/tee/management',
 'tee.management.commands': 'src/tee/management/commands',
 'tee.migrations': 'src/tee/migrations',
 'test_bpp': 'src/test_bpp',
 'test_bpp.migrations': 'src/test_bpp/migrations',
 'zglos_publikacje': 'src/zglos_publikacje',
 'zglos_publikacje.admin': 'src/zglos_publikacje/admin',
 'zglos_publikacje.migrations': 'src/zglos_publikacje/migrations',
 'zglos_publikacje.tests': 'src/zglos_publikacje/tests',
 'zglos_publikacje.tests.test_admin': 'src/zglos_publikacje/tests/test_admin'}

packages = \
['api_v1',
 'api_v1.serializers',
 'api_v1.tests',
 'api_v1.viewsets',
 'bpp',
 'bpp.admin',
 'bpp.admin.xlsx_export',
 'bpp.context_processors',
 'bpp.fixtures',
 'bpp.forms',
 'bpp.imports',
 'bpp.imports.uml',
 'bpp.management',
 'bpp.management.commands',
 'bpp.migrations',
 'bpp.models',
 'bpp.models.sloty',
 'bpp.multiseek_registry',
 'bpp.reports',
 'bpp.templatetags',
 'bpp.tests',
 'bpp.tests.test_admin',
 'bpp.tests.test_api',
 'bpp.tests.test_models',
 'bpp.tests.test_models.test_abstract',
 'bpp.tests.test_models.test_sloty',
 'bpp.tests.test_models.test_struktura',
 'bpp.tests.test_multiseek',
 'bpp.tests.test_selenium',
 'bpp.tests.test_selenium.test_admin',
 'bpp.tests.test_views',
 'bpp.tests.test_views.test_browse',
 'bpp.tests.tests_legacy',
 'bpp.tests.tests_legacy.test_reports',
 'bpp.tests.tests_legacy.test_views',
 'bpp.views',
 'bpp.views.api',
 'bpp.views.autocomplete',
 'bpp.views.raporty',
 'celeryui',
 'celeryui.migrations',
 'create_test_db',
 'create_test_db.management',
 'create_test_db.management.commands',
 'create_test_db.migrations',
 'crossref_bpp',
 'crossref_bpp.migrations',
 'crossref_bpp.tests',
 'django_bpp',
 'django_bpp.settings',
 'django_bpp.staticroot',
 'django_bpp.staticroot.notifications',
 'django_bpp.staticroot.notifications.js',
 'django_bpp.staticroot.notifications.js.tests',
 'dynamic_columns',
 'dynamic_columns.migrations',
 'ewaluacja2021',
 'ewaluacja2021.core',
 'ewaluacja2021.management',
 'ewaluacja2021.management.commands',
 'ewaluacja2021.migrations',
 'ewaluacja2021.tests',
 'ewaluacja2021.tests.test_core',
 'fixtures',
 'formdefaults',
 'formdefaults.migrations',
 'formdefaults.tests',
 'import_common',
 'import_common.tests',
 'import_dbf',
 'import_dbf.management',
 'import_dbf.management.commands',
 'import_dbf.migrations',
 'import_dbf.tests',
 'import_dyscyplin',
 'import_dyscyplin.management.commands',
 'import_dyscyplin.migrations',
 'import_dyscyplin.tests',
 'import_list_if',
 'import_list_if.migrations',
 'import_list_if.tests',
 'import_pracownikow',
 'import_pracownikow.migrations',
 'import_pracownikow.tests',
 'import_pracownikow.tests.test_models',
 'integrator2',
 'integrator2.migrations',
 'integrator2.models',
 'integrator2.tests',
 'long_running',
 'long_running.migrations',
 'long_running.tests',
 'miniblog',
 'miniblog.migrations',
 'notifications',
 'notifications.management',
 'notifications.management.commands',
 'notifications.migrations',
 'notifications.static',
 'notifications.static.notifications',
 'notifications.static.notifications.js',
 'notifications.static.notifications.js.tests',
 'notifications.templatetags',
 'notifications.tests',
 'nowe_raporty',
 'nowe_raporty.tests',
 'pbn_api',
 'pbn_api.adapters',
 'pbn_api.admin',
 'pbn_api.conf',
 'pbn_api.integrator',
 'pbn_api.management',
 'pbn_api.management.commands',
 'pbn_api.migrations',
 'pbn_api.models',
 'pbn_api.tests',
 'raport_slotow',
 'raport_slotow.migrations',
 'raport_slotow.models',
 'raport_slotow.tests',
 'raport_slotow.tests.test_raport_slotow_autor',
 'raport_slotow.tests.test_views',
 'raport_slotow.tests.tests_models',
 'raport_slotow.views',
 'rozbieznosci_dyscyplin',
 'rozbieznosci_dyscyplin.migrations',
 'rozbieznosci_if',
 'rozbieznosci_if.migrations',
 'tee',
 'tee.management',
 'tee.management.commands',
 'tee.migrations',
 'test_bpp',
 'test_bpp.migrations',
 'zglos_publikacje',
 'zglos_publikacje.admin',
 'zglos_publikacje.migrations',
 'zglos_publikacje.tests',
 'zglos_publikacje.tests.test_admin']

package_data = \
{'': ['*'],
 'bpp': ['static/admin/js/*',
         'static/admin/js/vendor/jquery/*',
         'static/adminsortable2/js/*',
         'static/autocomplete_light/i18n/*',
         'static/bpp/css/*',
         'static/bpp/flick/*',
         'static/bpp/flick/images/*',
         'static/bpp/images/*',
         'static/bpp/js/*',
         'static/bpp/svg/*',
         'static/bpp/wav/*',
         'static/grappelli/jquery/i18n/*',
         'static/scss/_settings.scss',
         'static/scss/_settings.scss',
         'static/scss/_settings.scss',
         'static/scss/_settings.scss',
         'static/scss/_settings.scss',
         'static/scss/_settings.scss',
         'static/scss/_settings.scss',
         'static/scss/_settings.scss',
         'static/scss/_settings_green.scss',
         'static/scss/_settings_green.scss',
         'static/scss/_settings_green.scss',
         'static/scss/_settings_green.scss',
         'static/scss/_settings_green.scss',
         'static/scss/_settings_green.scss',
         'static/scss/_settings_green.scss',
         'static/scss/_settings_green.scss',
         'static/scss/_settings_orange.scss',
         'static/scss/_settings_orange.scss',
         'static/scss/_settings_orange.scss',
         'static/scss/_settings_orange.scss',
         'static/scss/_settings_orange.scss',
         'static/scss/_settings_orange.scss',
         'static/scss/_settings_orange.scss',
         'static/scss/_settings_orange.scss',
         'static/scss/app-blue.scss',
         'static/scss/app-blue.scss',
         'static/scss/app-blue.scss',
         'static/scss/app-blue.scss',
         'static/scss/app-blue.scss',
         'static/scss/app-blue.scss',
         'static/scss/app-blue.scss',
         'static/scss/app-blue.scss',
         'static/scss/app-green.scss',
         'static/scss/app-green.scss',
         'static/scss/app-green.scss',
         'static/scss/app-green.scss',
         'static/scss/app-green.scss',
         'static/scss/app-green.scss',
         'static/scss/app-green.scss',
         'static/scss/app-green.scss',
         'static/scss/app-orange.scss',
         'static/scss/app-orange.scss',
         'static/scss/app-orange.scss',
         'static/scss/app-orange.scss',
         'static/scss/app-orange.scss',
         'static/scss/app-orange.scss',
         'static/scss/app-orange.scss',
         'static/scss/app-orange.scss',
         'static/scss/checkbox.scss',
         'static/scss/checkbox.scss',
         'static/scss/checkbox.scss',
         'static/scss/checkbox.scss',
         'static/scss/checkbox.scss',
         'static/scss/checkbox.scss',
         'static/scss/checkbox.scss',
         'static/scss/checkbox.scss',
         'static/scss/common.scss',
         'static/scss/common.scss',
         'static/scss/common.scss',
         'static/scss/common.scss',
         'static/scss/common.scss',
         'static/scss/common.scss',
         'static/scss/common.scss',
         'static/scss/common.scss',
         'templates/*',
         'templates/admin/*',
         'templates/autocompletes/*',
         'templates/browse/*',
         'templates/raporty/*',
         'templates/raporty/raport_jednostek_autorow_2012/*',
         'templates/raporty/raport_komisji_centralnej/*',
         'templates/raporty/strona_raportow/*'],
 'bpp.tests.test_admin': ['cassettes/test_crossref_api_helpers/*',
                          'cassettes/test_crossref_api_sync/*'],
 'celeryui': ['locale/pl/LC_MESSAGES/*'],
 'crossref_bpp.tests': ['cassettes/test_forms/*', 'cassettes/test_views/*'],
 'django_bpp': ['templates/*',
                'templates/admin/*',
                'templates/admin/bpp/autor/*',
                'templates/admin/bpp/bppuser/*',
                'templates/admin/bpp/patent/*',
                'templates/admin/bpp/szablondlaopisubibliograficznego/*',
                'templates/admin/bpp/wydawca/*',
                'templates/admin/bpp/wydawnictwo_ciagle/*',
                'templates/admin/bpp/wydawnictwo_ciagle_autor/*',
                'templates/admin/bpp/wydawnictwo_zwarte/*',
                'templates/admin/bpp/wydawnictwo_zwarte_autor/*',
                'templates/admin/bpp/zrodlo/*',
                'templates/admin/dbtemplates/template/*',
                'templates/adminsortable2/*',
                'templates/cookielaw/*',
                'templates/multiseek/*',
                'templates/pagination/*',
                'templates/permissions_widget/*',
                'templates/registration/*',
                'templates/rest_framework/*'],
 'django_bpp.staticroot': ['CACHE-202210.1089/*',
                           'CACHE-202210.1089/css/*',
                           'CACHE-202210.1089/js/*',
                           'admin/css/*',
                           'admin/css/vendor/select2/*',
                           'admin/fonts/*',
                           'admin/img/*',
                           'admin/img/gis/*',
                           'admin/js/*',
                           'admin/js/admin/*',
                           'admin/js/vendor/jquery/*',
                           'admin/js/vendor/select2/*',
                           'admin/js/vendor/select2/i18n/*',
                           'admin/js/vendor/xregexp/*',
                           'admin_tools/css/*',
                           'admin_tools/css/jquery/*',
                           'admin_tools/css/jquery/images/*',
                           'admin_tools/images/*',
                           'admin_tools/js/*',
                           'admin_tools/js/jquery/*',
                           'adminsortable2/css/*',
                           'adminsortable2/icons/*',
                           'adminsortable2/js/*',
                           'adminsortable2/js/libs/*',
                           'adminsortable2/js/plugins/*',
                           'autocomplete_light/*',
                           'autocomplete_light/i18n/*',
                           'bpp/css/*',
                           'bpp/flick/*',
                           'bpp/flick/images/*',
                           'bpp/images/*',
                           'bpp/js/*',
                           'bpp/svg/*',
                           'bpp/wav/*',
                           'cookielaw/css/*',
                           'cookielaw/img/*',
                           'cookielaw/js/*',
                           'css/*',
                           'datatables.net-zf/css/*',
                           'datatables.net-zf/js/*',
                           'datatables.net/js/*',
                           'dbtemplates/css/*',
                           'dbtemplates/js/*',
                           'django_extensions/css/*',
                           'django_extensions/img/*',
                           'django_extensions/js/*',
                           'django_tables2/*',
                           'django_tables2/themes/paleblue/css/*',
                           'django_tables2/themes/paleblue/img/*',
                           'djangoql/css/*',
                           'djangoql/img/*',
                           'djangoql/js/*',
                           'ewaluacja2021/xlsx/*',
                           'foundation-datepicker/css/*',
                           'foundation-datepicker/foundation/fonts/*',
                           'foundation-datepicker/foundation/fonts/svgs/*',
                           'foundation-datepicker/js/*',
                           'foundation-datepicker/js/locales/*',
                           'foundation-sites/dist/css/*',
                           'foundation-sites/dist/js/*',
                           'grappelli/images/*',
                           'grappelli/images/backgrounds/*',
                           'grappelli/images/icons-small/*',
                           'grappelli/images/icons/*',
                           'grappelli/img/admin/*',
                           'grappelli/jquery/*',
                           'grappelli/jquery/i18n/*',
                           'grappelli/jquery/ui/*',
                           'grappelli/jquery/ui/external/jquery/*',
                           'grappelli/jquery/ui/images/*',
                           'grappelli/js/*',
                           'grappelli/stylesheets/*',
                           'grappelli/stylesheets/mueller/*',
                           'grappelli/stylesheets/mueller/grid/*',
                           'grappelli/stylesheets/partials/custom/*',
                           'grappelli/tinymce_setup/*',
                           'import_dyscyplin/xlsx/*',
                           'import_export/*',
                           'import_list_if/*',
                           'import_pracownikow/*',
                           'jinplace/js/*',
                           'jquery.cookie/*',
                           'jquery/dist/*',
                           'jqueryui/*',
                           'js/*',
                           'js/crispy_forms_foundation/*',
                           'kbw-keypad/dist/*',
                           'kbw-keypad/dist/css/*',
                           'kbw-keypad/dist/img/*',
                           'kbw-keypad/dist/js/*',
                           'mptt/*',
                           'multiseek/css/*',
                           'multiseek/js/*',
                           'rest_framework/css/*',
                           'rest_framework/fonts/*',
                           'rest_framework/img/*',
                           'rest_framework/js/*',
                           'scss/*',
                           'select2-foundation_theme/dist/*',
                           'select2/dist/css/*',
                           'select2/dist/js/*',
                           'select2/dist/js/i18n/*',
                           'session_security/*',
                           'vendor/select2/dist/css/*',
                           'what-input/dist/*'],
 'dynamic_columns': ['locale/pl/LC_MESSAGES/django.po'],
 'ewaluacja2021': ['static/ewaluacja2021/xlsx/*', 'templates/ewaluacja2021/*'],
 'import_dbf': ['sql/*'],
 'import_dyscyplin': ['static/import_dyscyplin/xlsx/*',
                      'templates/import_dyscyplin/*'],
 'import_list_if': ['static/import_list_if/*', 'templates/import_list_if/*'],
 'import_pracownikow': ['static/import_pracownikow/*',
                        'templates/import_pracownikow/*'],
 'integrator2': ['templates/*', 'templates/integrator2/*'],
 'integrator2.tests': ['xls/*'],
 'long_running': ['templates/long_running/*'],
 'miniblog': ['locale/pl/LC_MESSAGES/django.po'],
 'nowe_raporty': ['fixtures/*', 'templates/nowe_raporty/*'],
 'pbn_api': ['templates/admin/pbn_api/institution/*',
             'templates/admin/pbn_api/journal/*',
             'templates/admin/pbn_api/oswiadczenieinstytucji/*',
             'templates/admin/pbn_api/publication/*',
             'templates/admin/pbn_api/publikacjainstytucji/*',
             'templates/admin/pbn_api/publisher/*',
             'templates/admin/pbn_api/scientist/*',
             'templates/admin/pbn_api/sentdata/*'],
 'raport_slotow': ['templates/raport_slotow/*'],
 'rozbieznosci_dyscyplin': ['templates/admin/rozbieznosci_dyscyplin/rozbieznosciview/*',
                            'templates/admin/rozbieznosci_dyscyplin/rozbieznoscizrodelview/*',
                            'templates/rozbieznosci_dyscyplin/*'],
 'rozbieznosci_if': ['templates/rozbieznosci_if/*'],
 'zglos_publikacje': ['templates/admin/zglos_publikacje/zgloszenie_publikacji/*',
                      'templates/templated_email/*',
                      'templates/zglos_publikacje/*']}

install_requires = \
['Django>=3.2,<3.3',
 'MOAI-iplweb==2.0.0',
 'Markdown==3.2.1',
 'Pillow==9.2.0',
 'Unidecode==0.4.20',
 'arrow==1.2.1',
 'bleach>=3.3.0',
 'celery[redis]==5.2.7',
 'channels-redis==3.4.0',
 'channels==3.0.5',
 'crispy-forms-foundation==0.8.0',
 'crossrefapi>=1.5.0,<2.0.0',
 'cssmin==0.2.0',
 'dbfread>=2.0.7,<3.0.0',
 'dj-pagination==2.4.0',
 'django-admin-sortable2==1.0',
 'django-admin-tools==0.9.0',
 'django-auth-ldap>=4.1.0,<5.0.0',
 'django-autocomplete-light==3.9.4',
 'django-autoslug==1.9.8',
 'django-braces>=1.15.0,<2.0.0',
 'django-cacheops==6.1',
 'django-celery-email>=3.0.0,<4.0.0',
 'django-celery-results==2.4.0',
 'django-classy-tags==1.0.0',
 'django-columns==0.1.0',
 'django-compressor==2.4',
 'django-cookie-law==2.0.3',
 'django-crispy-forms==1.9.0',
 'django-dbtemplates-iplweb==4.1.0',
 'django-denorm-iplweb==1.1.0',
 'django-dirtyfields==1.3.1',
 'django-easy-audit>=1.3.3,<2.0.0',
 'django-environ>=0.9.0,<0.10.0',
 'django-extensions==3.1.3',
 'django-favicon-plus==0.0.8',
 'django-filter==21.1',
 'django-flexible-reports>0.2.9',
 'django-formtools>=2.3,<3.0',
 'django-fsm==2.8.0',
 'django-grappelli==2.15.5',
 'django-group-by==0.3.1',
 'django-htmlmin==0.11.0',
 'django-import-export==2.5.0',
 'django-loginas==0.3.8',
 'django-messages-extends==0.6.2',
 'django-model-utils==4.0.0',
 'django-mptt==0.13.4',
 'django-multiseek==0.9.47',
 'django-password-policies-iplweb==0.8.4',
 'django-permissions-widget==1.5.2',
 'django-querysetsequence==0.12',
 'django-redis-cache==2.1.1',
 'django-reversion==4.0.0',
 'django-robots==4.0',
 'django-sendfile==0.3.11',
 'django-session-security-iplweb==2.6.6',
 'django-static-sitemaps>=4.7.0,<5.0.0',
 'django-tables2==2.3.1',
 'django-taggit-serializer==0.1.7',
 'django-taggit==3.0.0',
 'django-templated-email>=3.0.0,<4.0.0',
 'django-weasyprint>=2.1.0,<3.0.0',
 'django-webmaster-verification==0.3.0',
 'djangoql==0.15.3',
 'djangorestframework==3.11.2',
 'olefile==0.44',
 'openpyxl==3.0.10',
 'pathspec>=0.9.0,<0.10.0',
 'progressbar2>=4.0.0,<5.0.0',
 'psycopg2==2.8.6',
 'pygad==2.16.1',
 'pymed-iplweb==0.8.10',
 'pyoai==2.5.0',
 'pypandoc==1.5',
 'requests-oauthlib==1.3.0',
 'requests==2.26.0',
 'rjsmin==1.1.0',
 'sentry-sdk==1.9.2',
 'simplejson==3.17.5',
 'tablib==1.1.0',
 'uvicorn[standard]==0.18.2',
 'weasyprint==55.0',
 'wosclient==0.1.5',
 'xlrd==2.0.1',
 'zope.component==4.6.2',
 'zope.interface==5.1.0']

entry_points = \
{'console_scripts': ['bpp-manage.py = django_bpp.manage_command:entry_point']}

setup_kwargs = {
    'name': 'bpp-iplweb',
    'version': '202210.1089',
    'description': '',
    'long_description': 'None',
    'author': 'Michał Pasternak',
    'author_email': 'michal.dtz@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<3.11',
}


setup(**setup_kwargs)



# Generated by Django 3.0.14 on 2022-02-27 19:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0310_fix_refresh_cache_deadlocks"),
    ]

    operations = [
        migrations.AddField(
            model_name="wydawnictwo_ciagle",
            name="kwartyl_w_scopus",
            field=models.IntegerField(
                blank=True,
                choices=[(None, "brak"), (1, "Q1"), (2, "Q2"), (3, "Q3"), (4, "Q4")],
                default=None,
                null=True,
                verbose_name="Kwartyl w SCOPUS",
            ),
        ),
        migrations.AddField(
            model_name="wydawnictwo_ciagle",
            name="kwartyl_w_wos",
            field=models.IntegerField(
                blank=True,
                choices=[(None, "brak"), (1, "Q1"), (2, "Q2"), (3, "Q3"), (4, "Q4")],
                default=None,
                null=True,
                verbose_name="Kwartyl w WoS",
            ),
        ),
    ]



from crispy_forms.helper import FormHelper
from crispy_forms.layout import ButtonHolder, Submit
from crispy_forms_foundation.layout import Column, Fieldset, Layout, Row
from django import forms

from bpp.util import formdefaults_html_after, formdefaults_html_before
from import_list_if.models import ImportListIf


class NowyImportForm(forms.ModelForm):
    class Meta:
        model = ImportListIf
        fields = ["plik_xls", "rok"]

    def __init__(self, *args, **kwargs):
        self.helper = FormHelper()
        self.helper.form_class = "custom"
        self.helper.form_action = "."
        self.helper.layout = Layout(
            Fieldset(
                "Wybierz parametry",
                formdefaults_html_before(self),
                Row(
                    Column("plik_xls", css_class="large-12 small-12"),
                    Column("rok", css_class="large-12 small-12"),
                ),
                formdefaults_html_after(self),
            ),
            ButtonHolder(
                Submit(
                    "submit",
                    "Utwórz import",
                    css_id="id_submit",
                    css_class="submit button",
                ),
            ),
        )

        super(NowyImportForm, self).__init__(*args, **kwargs)



# -*- encoding: utf-8 -*
from optparse import make_option

from notifications.core import send_notification

try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse
from django.db import transaction

import messages_extends as messages
from django.contrib.auth import get_user_model

from django.core.management import BaseCommand
from django.test import RequestFactory
from messages_extends.storages import PersistentStorage

import notifications
from messages_extends.models import Message


class Command(BaseCommand):
    help = "Wysyla komunikat offline za pomoca frameworku messages"
    args = "<username> <message>"

    def add_arguments(self, parser):
        parser.add_argument("username")
        parser.add_argument("text")

        parser.add_argument(
            "--dont-persist",
            action="store_true",
            default=False,
            help="Don't persist the message",
        )

    def handle(self, username, text, dont_persist, *args, **options):
        request_factory = RequestFactory()
        request = request_factory.get("/")

        request.user = get_user_model().objects.get(username=username)
        setattr(request, "session", "session")

        storage = PersistentStorage(request)

        setattr(request, "_messages", storage)

        level = messages.INFO_PERSISTENT

        msg = None

        if dont_persist:
            send_notification(
                request, level, text,
            )
            return

        with transaction.atomic():
            messages.add_message(request, level, text)
            msg = Message.objects.filter(
                user_id=request.user.pk, message=text
            ).order_by("-pk")[:1]

        if msg:
            msg = msg[0]
            closeURL = reverse("messages_extends:message_mark_read", args=(msg.pk,))
            send_notification(
                request, level, text, closeURL=closeURL,
            )



# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0037_auto_20160124_1336'),
    ]

    operations = [
        migrations.AddField(
            model_name='wydawnictwo_ciagle',
            name='liczba_znakow_wydawniczych',
            field=models.IntegerField(db_index=True, null=True, verbose_name=b'Liczba znak\xc3\xb3w wydawniczych', blank=True),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='liczba_znakow_wydawniczych',
            field=models.IntegerField(db_index=True, null=True, verbose_name=b'Liczba znak\xc3\xb3w wydawniczych', blank=True),
        ),
    ]



class ZrodloPBNAdapter:
    def __init__(self, original):
        self.original = original

    def pbn_get_json(self):
        if self.original.pbn_uid_id is None:
            ret = {"title": self.original.nazwa}
            if self.original.wydawca:
                ret["publisher"] = {"name": self.original.wydawca}
            if self.original.issn:
                ret["issn"] = self.original.issn
            if self.original.www:
                ret["websiteLink"] = self.original.www
            if self.original.e_issn:
                ret["eissn"] = self.original.e_issn
            return ret

            #     "issue": {
            #       "doi": "string",
            #       "number": "string",
            #       "objectId": "string",
            #       "publishedYear": 0,
            #       "versionHash": "string",
            #       "volume": "string",
            #       "year": "string"
            #     },
            #     "mniswId": 0,
            #     "objectId": "string",

        #  "journal": {
        #     "eissn": "string",
        #     "issn": "string",
        #     "issue": {
        #       "doi": "string",
        #       "number": "string",
        #       "objectId": "string",
        #       "publishedYear": 0,
        #       "versionHash": "string",
        #       "volume": "string",
        #       "year": "string"
        #     },
        #     "publisher": {
        #       "mniswId": 0,
        #       "name": "string",
        #       "objectId": "string",
        #       "versionHash": "string"
        #     },
        #     "title": "string",
        #     "versionHash": "string",
        #     "websiteLink": "string"
        #   },

        journal = self.original.pbn_uid

        ret = {
            "objectId": journal.pk,
        }

        for attr in [
            "eissn",
            "issn",
            "publisher",
            "title",
            "websiteLink",
            "mniswId",
        ]:
            v = journal.value("object", attr, return_none=True)
            if v is not None:
                ret[attr] = v

        return ret



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-09 10:40
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('import_dyscyplin', '0003_auto_20180409_1129'),
    ]

    operations = [
        migrations.RenameField(
            model_name='import_dyscyplin_row',
            old_name='autor_id',
            new_name='autor',
        ),
        migrations.RenameField(
            model_name='import_dyscyplin_row',
            old_name='jednostka_id',
            new_name='jednostka',
        ),
        migrations.RenameField(
            model_name='import_dyscyplin_row',
            old_name='wydzial_id',
            new_name='wydzial',
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0013_wydzial_zezwalaj_na_ranking_autorow'),
    ]

    operations = [
        migrations.AlterField(
            model_name='wydzial',
            name='widoczny',
            field=models.BooleanField(default=True, help_text=b'Czy wydzia\xc5\x82 ma by\xc4\x87 widoczny przy przegl\xc4\x85daniu strony dla zak\xc5\x82adki "Uczelnia"?'),
            preserve_default=True,
        ),
    ]



# Create your views here.
import operator
import os
from functools import reduce

from django.conf import settings
from django.core.exceptions import ValidationError
from django.core.files.storage import FileSystemStorage
from django.db import transaction
from django.http import Http404
from django.shortcuts import render
from django.views.generic import TemplateView
from formtools.wizard.views import SessionWizardView
from messages_extends import messages
from sentry_sdk import capture_exception
from templated_email import send_templated_mail

from import_common.normalization import normalize_tytul_publikacji
from zglos_publikacje import const
from zglos_publikacje.forms import (
    Zgloszenie_Publikacji_AutorFormSet,
    Zgloszenie_Publikacji_DaneOgolneForm,
    Zgloszenie_Publikacji_KosztPublikacjiForm,
    Zgloszenie_Publikacji_Plik,
)
from zglos_publikacje.models import (
    Obslugujacy_Zgloszenia_Wydzialow,
    Zgloszenie_Publikacji,
)

from bpp.const import PUSTY_ADRES_EMAIL, TO_AUTOR
from bpp.core import zgloszenia_publikacji_emails
from bpp.models import Typ_Odpowiedzialnosci, Uczelnia
from bpp.views.mixins import UczelniaSettingRequiredMixin


class Sukces(TemplateView):
    template_name = "zglos_publikacje/sukces.html"


def pokazuj_formularz_pliku(wizard):
    """Jeżeli w pierwszym kroku podano prawidłowy adres URL dla strony WWW, to nie pytaj
    o plik. Jeżeli nie podano - pytaj."""
    cleaned_data = wizard.get_cleaned_data_for_step("0") or {}
    return not cleaned_data.get("strona_www", None)


def pokazuj_formularz_platnosci(wizard):
    # Jeżeli dla uczelni wyłączono potrzebę wpisywania informacji o płatnościach to nie pokazuj
    # tego formularza:

    uczelnia = Uczelnia.objects.get_for_request(wizard.request)
    if uczelnia is not None:
        if uczelnia.wymagaj_informacji_o_oplatach is not True:
            return False

    # OK, nie wyłączono globalnie podawania informacji o opłaach

    # Jeżeli w pierwszym kroku podano rodzaj publikacji jako artykuł naukowy lub monografia
    # to zapytaj o koszta. Jeżeli rodzaj jest inny -- to nie pytaj
    cleaned_data = wizard.get_cleaned_data_for_step("0") or {}
    return (
        cleaned_data.get("rodzaj_zglaszanej_publikacji", None)
        == Zgloszenie_Publikacji.Rodzaje.ARTYKUL_LUB_MONOGRAFIA
    )


class Zgloszenie_PublikacjiWizard(UczelniaSettingRequiredMixin, SessionWizardView):
    uczelnia_attr = "pokazuj_formularz_zglaszania_publikacji"

    template_name = "zglos_publikacje/zgloszenie_publikacji_form.html"
    file_storage = FileSystemStorage(
        location=os.path.join(settings.MEDIA_ROOT, "zglos_publikacje")
    )
    form_list = [
        Zgloszenie_Publikacji_DaneOgolneForm,
        Zgloszenie_Publikacji_Plik,
        Zgloszenie_Publikacji_AutorFormSet,
        Zgloszenie_Publikacji_KosztPublikacjiForm,
    ]
    condition_dict = {"1": pokazuj_formularz_pliku, "3": pokazuj_formularz_platnosci}

    object = None

    def get_form_instance(self, step):
        kod_do_edycji = self.kwargs.get("kod_do_edycji")
        if kod_do_edycji:
            try:
                self.object = Zgloszenie_Publikacji.objects.get(
                    kod_do_edycji=kod_do_edycji
                )
            except Zgloszenie_Publikacji.DoesNotExist:
                raise Http404

            return {
                "0": self.object,
                "1": self.object,
                "2": self.object,
                "3": self.object,
            }[step]

    def process_step(self, form):
        if self.steps.current == "0":
            # Dla pierwszego formularza zapisz wartość roku w sesji:
            self.request.session[const.SESSION_KEY] = form.cleaned_data.get("rok")
        return super().process_step(form)

    def get_context_data(self, form, **kwargs):
        if self.request.session.get(const.SESSION_KEY):
            # Jeżeli wartość roku jest w sesji, to zwróć go do kontekstu:
            kwargs["rok"] = self.request.session.get(const.SESSION_KEY)
        return super().get_context_data(form, **kwargs)

    def get_form_initial(self, step):
        # Dla kroku "0" jeżeli użytkownik ma poprawny e-mail, to go użyj:
        if step == "0":
            if self.request.user.is_authenticated:
                if self.request.user.email != PUSTY_ADRES_EMAIL:
                    return {"email": self.request.user.email}

        # Dla kroku "2" (autorzy, dyscypliny) wstaw parametr rok:
        if step == "2":
            return [{"rok": self.request.session.get(const.SESSION_KEY)}] * 512
        return super().get_form_initial(step)

    @transaction.atomic
    def done(self, form_list, **kwargs):
        dane_rekordu = form_list[0].cleaned_data

        rest_of_data = {}

        if dane_rekordu.get("strona_www"):
            # Dodający podał stronę WWW --  wiec nie było pytania o plik -- wiec formularz [1] to
            # lista autorów, a pozostałe formularze zaczną się od [2]
            autorset_form_list = 1

            pozostale_formularze = [form.cleaned_data for form in form_list[2:]]
            if pozostale_formularze:
                rest_of_data = reduce(operator.or_, pozostale_formularze)
        else:
            # Dodający NIE podał strony WWW, czyli formularz [1] zawiera dane o pliku,
            # [2] to lista autorów a formularz [3] i kolejne...
            autorset_form_list = 2

            rest_of_data = reduce(
                operator.or_,
                [
                    form_list[1].cleaned_data,
                ]
                + [form.cleaned_data for form in form_list[3:]],
            )

        kwargs = dane_rekordu | rest_of_data

        if self.object is None:
            self.object = Zgloszenie_Publikacji(
                status=Zgloszenie_Publikacji.Statusy.NOWY
            )
        else:
            # Jezeli obiekt już istniał w bazie, to oznacza, że jest edytowany przez zgłaszającego
            # czyli należy dać mu status PO_ZMIANACH:
            self.object.status = Zgloszenie_Publikacji.Statusy.PO_ZMIANACH

            # Ustaw kod_do_edycji na pusty; jeżeli obiekt jest edytowany, to wyczyszczenie tego pola uniemożliwi
            # ponowne wejście w edycję.
            self.object.kod_do_edycji = None

            # Zresetuj przyczynę zwrotu -- rekord został zmodyfikowany
            self.object.przyczyna_zwrotu = None

        for attr_name, value in kwargs.items():
            setattr(self.object, attr_name, value)

        if self.request.user.is_authenticated and self.object.utworzyl_id is None:
            self.object.utworzyl = self.request.user

        if self.object.tytul_oryginalny:
            # Jeżeli jest tytuł oryginalny, to znormalizuj go, m.in. wycinając znaki
            # newline, ponieważ django-templated-email w wersji 3.0.0 nie obsługuje ich,
            # ma to poprawione w trunku, po nowym release można zaktualizować django-templated-email
            # i pozbyć się tego kodu
            #
            # https://github.com/vintasoftware/django-templated-email/issues/138

            self.object.tytul_oryginalny = normalize_tytul_publikacji(
                self.object.tytul_oryginalny
            )

        self.object.save()

        typ_odpowiedzialnosci = Typ_Odpowiedzialnosci.objects.filter(
            typ_ogolny=TO_AUTOR
        ).first()

        if typ_odpowiedzialnosci is None:
            raise ValidationError(
                "Nie można utworzyć danych -- w systemie nie jest skonfigurowany "
                "żaden typ odpowiedzialnosci z typem ogólnym = autor."
            )

        autorzy_formset = form_list[autorset_form_list]

        if autorzy_formset.is_valid():
            for no, form in enumerate(autorzy_formset):
                if form.is_valid():
                    if form.cleaned_data.get("DELETE"):
                        if form.instance.pk:
                            form.instance.delete()
                        continue
                    else:
                        if not form.cleaned_data:
                            # Formularz może być zupełnie pusty
                            continue

                        instance = form.save(commit=False)
                        instance.rekord = self.object
                        instance.kolejnosc = no
                        instance.typ_odpowiedzialnosci = typ_odpowiedzialnosci
                        instance.save()

        def _():
            recipient_list = None

            # Wybór autora i jednostki.
            #
            # Szukamy pierwszej, nie-obcej jednostki, skupiającej pracowników.
            # Jeżeli nie znajdziemy takiej, używamy obcej.

            jednostka_do_powiadomienia = None

            for zpa in self.object.zgloszenie_publikacji_autor_set.all().select_related(
                "jednostka"
            ):
                if zpa.jednostka_id is not None:
                    jednostka_do_powiadomienia = zpa.jednostka

                if jednostka_do_powiadomienia.skupia_pracownikow:
                    break

            if jednostka_do_powiadomienia is not None:
                recipient_list = (
                    Obslugujacy_Zgloszenia_Wydzialow.objects.emaile_dla_wydzialu(
                        jednostka_do_powiadomienia.wydzial
                    )
                )

            if not recipient_list:
                recipient_list = zgloszenia_publikacji_emails()

            try:
                send_templated_mail(
                    template_name="nowe_zgloszenie",
                    from_email=getattr(
                        settings, "DEFAULT_FROM_EMAIL", "webmaster@localhost"
                    ),
                    headers={"reply-to": self.object.email},
                    recipient_list=recipient_list,
                    context={
                        "object": self.object,
                        "site_url": self.request.get_host(),
                    },
                )
            except Exception as e:
                capture_exception(e)

                messages.add_message(
                    self.request,
                    messages.WARNING,
                    "Z uwagi na błąd wysyłania komunikatu z powiadomieniem, zespół Biblioteki nie "
                    "został powiadomiony o dodaniu zgłoszenia. Prosimy wysłać e-mail bądź skontaktować się drogą "
                    "telefoniczną.",
                )

        transaction.on_commit(_)

        return render(
            self.request,
            "zglos_publikacje/sukces.html",
            {
                "form_data": [form.cleaned_data for form in form_list],
                "object": self.object,
            },
        )



# Generated by Django 3.0.11 on 2021-04-06 03:30

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("pbn_api", "0007_publication"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="Sciencist",
            new_name="Scientist",
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-06-12 22:22
from __future__ import unicode_literals

import bpp.models.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0137_auto_20180613_0015'),
    ]

    operations = [
        migrations.AddField(
            model_name='uczelnia',
            name='pokazuj_liczbe_cytowan_w_rankingu',
            field=bpp.models.fields.OpcjaWyswietlaniaField(choices=[('always', 'zawsze'), ('logged-in', 'tylko dla zalogowanych'), ('never', 'nigdy')], default='always', max_length=50, verbose_name='Pokazuj liczbę cytowań w rankingu'),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0029_auto_20150923_1343'),
    ]

    operations = [
        migrations.AddField(
            model_name='zrodlo',
            name='openaccess_tryb_dostepu',
            field=models.CharField(blank=True, max_length=50, verbose_name=b'OpenAccess: tryb dost\xc4\x99pu', db_index=True, choices=[(b'FULL', b'pe\xc5\x82ny'), (b'PARTIAL', b'cz\xc4\x99\xc5\x9bciowy')]),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle',
            name='openaccess_czas_publikacji',
            field=models.ForeignKey(on_delete=models.CASCADE, verbose_name=b'OpenAccess: czas udost\xc4\x99pnienia', blank=True, to='bpp.Czas_Udostepnienia_OpenAccess', null=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle',
            name='openaccess_licencja',
            field=models.ForeignKey(on_delete=models.CASCADE, verbose_name=b'OpenAccess: licencja', blank=True, to='bpp.Licencja_OpenAccess', null=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle',
            name='openaccess_tryb_dostepu',
            field=models.ForeignKey(on_delete=models.CASCADE, verbose_name=b'OpenAccess: tryb dost\xc4\x99pu', blank=True, to='bpp.Tryb_OpenAccess_Wydawnictwo_Ciagle', null=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle',
            name='openaccess_wersja_tekstu',
            field=models.ForeignKey(on_delete=models.CASCADE, verbose_name=b'OpenAccess: wersja tekstu', blank=True, to='bpp.Wersja_Tekstu_OpenAccess', null=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='openaccess_czas_publikacji',
            field=models.ForeignKey(on_delete=models.CASCADE, verbose_name=b'OpenAccess: czas udost\xc4\x99pnienia', blank=True, to='bpp.Czas_Udostepnienia_OpenAccess', null=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='openaccess_licencja',
            field=models.ForeignKey(on_delete=models.CASCADE, verbose_name=b'OpenAccess: licencja', blank=True, to='bpp.Licencja_OpenAccess', null=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='openaccess_wersja_tekstu',
            field=models.ForeignKey(on_delete=models.CASCADE, verbose_name=b'OpenAccess: wersja tekstu', blank=True, to='bpp.Wersja_Tekstu_OpenAccess', null=True),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0009_auto_20141126_1804'),
    ]

    operations = [
        migrations.AlterField(
            model_name='charakter_formalny',
            name='nazwa_w_primo',
            field=models.CharField(default=b'', choices=[('', ''), ('Artyku\u0142', 'Artyku\u0142'), ('Ksi\u0105\u017cka', 'Ksi\u0105\u017cka'), ('Zas\xf3b tekstowy', 'Zas\xf3b tekstowy'), ('Rozprawa naukowa', 'Rozprawa naukowa'), ('Recenzja', 'Recenzja'), ('Artyku\u0142 prasowy', 'Artyku\u0142 prasowy'), ('Rozdzia\u0142', 'Rozdzia\u0142'), ('Czasopismo', 'Czasopismo'), ('Dane badawcze', 'Dane badawcze'), ('Materia\u0142 konferencyjny', 'Materia\u0142 konferencyjny'), ('Obraz', 'Obraz'), ('Baza', 'Baza'), ('Zestaw danych statystycznych', 'Zestaw danych statystycznych'), ('Multimedia', 'Multimedia'), ('Inny', 'Inny')], max_length=100, blank=True, help_text=b'\n    Nazwa charakteru formalnego w wyszukiwarce Primo, eksponowana przez OAI-PMH. W przypadku,\n    gdy to pole jest puste, prace o danym charakterze formalnym nie b\xc4\x99d\xc4\x85 udost\xc4\x99pniane przez\n    protok\xc3\xb3\xc5\x82 OAI-PMH.\n    ', verbose_name=b'Nazwa w Primo', db_index=True),
            preserve_default=True,
        ),
    ]



# -*- encoding: utf-8 -*-
from collections import defaultdict

from django.db import models

from bpp.models.abstract import ModelZAdnotacjami, ModelZNazwa


class Konferencja(ModelZNazwa, ModelZAdnotacjami):
    nazwa = models.TextField(max_length=512, db_index=True)

    skrocona_nazwa = models.CharField(
        "Skrócona nazwa", max_length=250, null=True, blank=True, db_index=True
    )

    rozpoczecie = models.DateField("Rozpoczęcie", null=True, blank=True)

    zakonczenie = models.DateField("Zakończenie", null=True, blank=True)

    miasto = models.CharField("Miasto", max_length=100, null=True, blank=True)

    panstwo = models.CharField("Państwo", max_length=100, null=True, blank=True)

    baza_scopus = models.BooleanField("Indeksowana w SCOPUS?", default=False)

    baza_wos = models.BooleanField("Indeksowana w WOS?", default=False)

    baza_inna = models.CharField(
        "Indeksowana w...",
        max_length=200,
        help_text="Wpisz listę innych baz czasopism i abstraktów, w których indeksowana "
        "była ta konferencja. Rozdziel średnikiem.",
        blank=True,
        null=True,
    )

    pbn_uid = models.ForeignKey(
        "pbn_api.Conference",
        verbose_name="Odpowiednik w PBN",
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
    )

    TK_KRAJOWA = 1
    TK_MIEDZYNARODOWA = 2
    TK_LOKALNA = 3

    TK_KRAJOWA_SYMBOL = "🚆 "
    TK_MIEDZYNARODOWA_SYMBOL = "✈ "
    TK_LOKALNA_SYMBOL = "🚲 "

    TK_SYMBOLE = defaultdict(
        lambda: "",
        {
            TK_KRAJOWA: TK_KRAJOWA_SYMBOL,
            TK_MIEDZYNARODOWA: TK_MIEDZYNARODOWA_SYMBOL,
            TK_LOKALNA: TK_LOKALNA_SYMBOL,
        },
    )

    TYP_KONFERENCJI_CHOICES = (
        (TK_KRAJOWA, TK_SYMBOLE[TK_KRAJOWA] + "krajowa"),
        (TK_MIEDZYNARODOWA, TK_SYMBOLE[TK_MIEDZYNARODOWA] + "️międzynarodowa"),
        (TK_LOKALNA, TK_SYMBOLE[TK_LOKALNA] + "lokalna"),
    )

    typ_konferencji = models.SmallIntegerField(
        choices=TYP_KONFERENCJI_CHOICES, null=True, blank=True
    )

    class Meta:
        unique_together = ("nazwa", "rozpoczecie")
        ordering = ("-rozpoczecie", "nazwa")
        verbose_name = "konferencja"
        verbose_name_plural = "konferencje"

    def __str__(self):
        ret = self.nazwa
        if self.baza_scopus:
            ret += " [Scopus]"
        if self.baza_wos:
            ret += " [WoS]"
        if self.baza_inna:
            ret += f" [{ self.baza_inna }]"
        return ret



from rest_framework import serializers


class ChoicesSerializerField(serializers.SerializerMethodField):
    def to_representation(self, value):
        method_name = "get_{field_name}_display".format(field_name=self.field_name)
        method = getattr(value, method_name)
        return method()


class AbsoluteUrlSerializerMixin(serializers.ModelSerializer):
    absolute_url = serializers.SerializerMethodField("get_absolute_url")

    def get_absolute_url(self, obj):
        return self.context["request"].build_absolute_uri(obj.get_absolute_url())


class AbsoluteUrlField(AbsoluteUrlSerializerMixin, serializers.RelatedField):
    def to_representation(self, value):
        if value is None:
            return
        return self.get_absolute_url(value)


class Wydawnictwo_AutorSerializerMixin(serializers.HyperlinkedModelSerializer):
    autor = serializers.HyperlinkedRelatedField(
        view_name="api_v1:autor-detail", read_only=True
    )

    jednostka = serializers.HyperlinkedRelatedField(
        view_name="api_v1:jednostka-detail", read_only=True
    )

    typ_odpowiedzialnosci = serializers.StringRelatedField()

    dyscyplina_naukowa = serializers.HyperlinkedRelatedField(
        view_name="api_v1:dyscyplina_naukowa-detail", read_only=True
    )


class WydawnictwoSerializerMixin(serializers.HyperlinkedModelSerializer):
    jezyk = serializers.HyperlinkedRelatedField(
        view_name="api_v1:jezyk-detail", read_only=True
    )
    jezyk_alt = serializers.HyperlinkedRelatedField(
        view_name="api_v1:jezyk-detail", read_only=True
    )
    charakter_formalny = serializers.HyperlinkedRelatedField(
        view_name="api_v1:charakter_formalny-detail", read_only=True
    )

    typ_kbn = serializers.HyperlinkedRelatedField(
        view_name="api_v1:typ_kbn-detail", read_only=True
    )
    status_korekty = serializers.StringRelatedField()

    openaccess_tryb_dostepu = serializers.StringRelatedField()
    openaccess_wersja_tekstu = serializers.StringRelatedField()
    openaccess_licencja = serializers.StringRelatedField()
    openaccess_czas_publikacji = serializers.HyperlinkedRelatedField(
        view_name="api_v1:czas_udostepnienia_openaccess-detail", read_only=True
    )

    nagrody = serializers.HyperlinkedRelatedField(
        many=True, view_name="api_v1:nagroda-detail", read_only=True,
    )



import itertools
import math
import os
import random
from operator import attrgetter

# import multiprocessing
import billiard as multiprocessing
import pygad

from .ewaluacja3n_base import Ewaluacja3NBase
from .genetyczny_multiprocessing import (
    FitnessFuncMixin,
    MultiprocessingGAD,
    multiprocessing_gad_pool_initializer,
)
from .util import splitEveryN


class GAD(FitnessFuncMixin, Ewaluacja3NBase):
    def __init__(
        self,
        nazwa_dyscypliny="nauki medyczne",
        max_gen=1000,
        saturate=500,
        ile_najlepszych=50,
        ile_losowych=100,
        ile_zupelnie_losowych=50,
        ile_epok=5,
        output_path=None,
    ):
        Ewaluacja3NBase.__init__(
            self=self, nazwa_dyscypliny=nazwa_dyscypliny, output_path=output_path
        )

        self.get_data()

        self.saturate = saturate

        self.max_gen = max_gen
        self.best_pkd = 0
        self.starting_pkd = 0

        self.maks_epok = ile_epok

        self.lista_prac_dict = {x.id: x for x in self.lista_prac_tuples}

        self.ile_najlepszych = ile_najlepszych
        self.ile_losowych = ile_losowych
        self.ile_zupelnie_losowych = ile_zupelnie_losowych

    def on_generation(self, ga_instance: pygad.GA):
        no_generations = ga_instance.generations_completed or 1
        best_solution = ga_instance.best_solution()

        print(
            f"Generacja: {no_generations}\t"
            f"Procent: {no_generations * 100 // self.max_gen} \t"
            f"Najlepsze rozwiązanie: {best_solution[1]}\t"
            "\r",
            end="",
        )

    def pracuj(self):

        self.lista_prac_by_index = {
            idx: praca for idx, praca in enumerate(self.lista_prac_tuples)
        }
        self.index_prac_by_id = {
            praca.id: idx for idx, praca in self.lista_prac_by_index.items()
        }

        self.najlepszy_osobnik = [
            self.index_prac_by_id[x.id]
            for x in sorted(
                self.lista_prac_tuples,
                key=lambda x: attrgetter("pkdaut")(x) / attrgetter("slot")(x),
                reverse=True,
            )
        ]

        no_workers = int(math.ceil(os.cpu_count() * 0.75))

        self.pool = multiprocessing.Pool(
            processes=no_workers,
            initializer=multiprocessing_gad_pool_initializer,
            initargs=(
                self.lista_prac_by_index,
                self.liczba_2_2_n,
                self.liczba_0_8_n,
                self.maks_pkt_aut_calosc,
                self.maks_pkt_aut_monografie,
            ),
        )

        def cykl_genetyczny(najlepszy_osobnik, nr_cyklu):

            osbn_czesciowo_losowe = []

            baza_dla_randomizera = najlepszy_osobnik

            # Generuje połowe osobnikow losowych
            # Tutaj osobnik losowy ma zamieniane geny losowo w zakresie od 20% do 80%
            dlugosc = len(baza_dla_randomizera)
            start = dlugosc // 5
            end = dlugosc * 3 // 5
            for a in range(self.ile_losowych // 2):
                start = random.randint(start, end)
                end = random.randint(start, dlugosc)
                osobnik = (
                    baza_dla_randomizera[:start]
                    + random.sample(baza_dla_randomizera[start:end], end - start)
                    + baza_dla_randomizera[end:]
                )
                osbn_czesciowo_losowe.append(osobnik)

            # Generuje druga połowe losowych
            # Tutaj osobnik losowy ma zamieniane segmenty genów losowo, tzn jest dzielony
            # na od 4 do 50 czesci i te czesci ma potem łączone w losowy sposób
            for a in range(self.ile_losowych // 2):
                num_parts = random.randint(2, 50)
                split_size = dlugosc // num_parts
                if split_size == 0:
                    split_size = 1

                parts = splitEveryN(split_size, najlepszy_osobnik)
                random.shuffle(parts)
                osobnik = list(itertools.chain(*parts))
                osbn_czesciowo_losowe.append(osobnik)

            osbn_losowe = [  # noqa
                random.sample(baza_dla_randomizera, len(baza_dla_randomizera))
                for n in range(self.ile_zupelnie_losowych)
            ]

            initial_population = (
                [najlepszy_osobnik] * self.ile_najlepszych
                + osbn_losowe
                + osbn_czesciowo_losowe
            )

            sum_all = sum(praca.pkdaut for praca in self.lista_prac_tuples)

            # mutation

            self.ga_instance = MultiprocessingGAD(
                pool=self.pool,
                num_generations=self.max_gen,
                num_parents_mating=2,
                initial_population=initial_population,
                gene_type=int,
                gene_space=baza_dla_randomizera,
                crossover_type="single_point",
                mutation_type="swap",
                # mutation_num_genes=int(math.ceil(len(najlepszy_osobnik) * 0.5)),
                on_generation=lambda s: self.on_generation(s),
                stop_criteria=[f"reach_{sum_all}", f"saturate_{self.saturate}"],
            )

            self.ga_instance.run()

            solution, fitness, idx = self.ga_instance.best_solution()

            print(f"\n\nNajlepsze rozwiązanie z algorytmu genetycznego: {fitness}")

            solution = solution.tolist()
            print("Przeprowadzam przesuwanie bąbelkowe...")
            direction = 1
            ile_razy_nie_znaleziono_lepszych = 0
            while True:

                znaleziono_lepsze = False

                for a in range(0, len(solution)):
                    if a == 0 and direction == 1:
                        continue

                    if a == len(solution) - 1 and direction == -1:
                        continue

                    new_solution = solution[:]
                    if direction == 1:
                        new_solution.insert(0, new_solution.pop(a))
                    else:
                        new_solution.append(new_solution.pop(a))

                    self.fitness_func(new_solution)

                    if self.suma_pkd > fitness:
                        fitness = self.suma_pkd
                        print(
                            f"Nowe rozwiązanie z przesuwania bąbelkowego: {fitness}\r",
                            end="",
                        )
                        solution = new_solution
                        ile_razy_nie_znaleziono_lepszych = 0
                        znaleziono_lepsze = True

                if not znaleziono_lepsze:
                    direction = -direction
                    ile_razy_nie_znaleziono_lepszych += 1

                    if ile_razy_nie_znaleziono_lepszych == 2:
                        # Nie znaleziono lepszych zestawów w obydwu kierunkach
                        break

            self.fitness_func(solution)
            print(f"Po przesuwaniu bąbelkowym: {self.suma_pkd}")

            return solution

        self.suma_pkd = 0
        ile_epok = 0

        solution = self.najlepszy_osobnik

        while ile_epok < self.maks_epok:  # and poprzedni_wynik < self.suma_pkd:
            ile_epok += 1
            print(f"Obliczam epoke nr {ile_epok}")
            solution = cykl_genetyczny(solution, nr_cyklu=ile_epok)
            self.fitness_func(solution)

        # self.fitness_func(solution)
        self.lista_prac = self.lista_prac_tuples

    def pozegnanie(self):
        print(
            "Number of generations passed is {generations_completed}".format(
                generations_completed=self.ga_instance.generations_completed
            )
        )

        print("Najlepsza punktacja:", self.ga_instance.best_solution()[1])



from django import forms


class ORCIDField(forms.CharField):
    def __init__(self, *args, **kwargs):
        kwargs["max_length"] = 19
        kwargs["min_length"] = 19
        super().__init__(*args, **kwargs)



"""
Struktura uczelni.
"""
from typing import TYPE_CHECKING, List, Union

from autoslug import AutoSlugField
from django.core.exceptions import ImproperlyConfigured, ValidationError
from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import ProgrammingError, models
from django.db.models import SET_NULL, Max
from django.urls.base import reverse
from model_utils import Choices

from pbn_api.exceptions import WillNotExportError
from .. import const
from ..const import GR_RAPORTY_WYSWIETLANIE
from ..util import year_last_month
from .fields import OpcjaWyswietlaniaField

from django.utils.functional import cached_property

from bpp.models import ModelZAdnotacjami, NazwaISkrot
from bpp.models.abstract import ModelZPBN_ID, NazwaWDopelniaczu

if TYPE_CHECKING:
    from pbn_api import client  # noqa


class UczelniaManager(models.Manager):
    def get_default(self) -> Union["Uczelnia", None]:
        try:
            return self.all().first()
        except ProgrammingError:
            # Błąd może wystapić w sytuacji, gdy do obiektu Uczelnia po stronie kodu zostały
            # dodane jakieś kolumny, ale nie zostały jeszcze dodane do bazy danych. Próba "grzebnięcia"
            # po bazie spowoduje błąd. Spróbujemy wobec tego pobrać wyłacznie PK rekordu:
            return self.all().only("pk").first()

    def get_for_request(self, request):
        if hasattr(request, "_uczelnia"):
            return request._uczelnia

        return self.get_default()

    @cached_property
    def default(self):
        return self.get_default()

    def do_roku_default(self=None, request=None):
        if self is None:
            # Uruchomione przez migrację
            return
        uczelnia = self.get_default()
        if (
            uczelnia is None
            or uczelnia.metoda_do_roku_formularze
            == const.DO_STYCZNIA_POPRZEDNI_POTEM_OBECNY
        ):
            return year_last_month()
        if uczelnia.metoda_do_roku_formularze == const.NAJWIEKSZY_REKORD:
            from bpp.models.cache import Rekord

            return Rekord.objects.all().aggregate(Max("rok"))["rok__max"]
        raise NotImplementedError


class Uczelnia(ModelZAdnotacjami, ModelZPBN_ID, NazwaISkrot, NazwaWDopelniaczu):
    slug = AutoSlugField(populate_from="skrot", unique=True)
    logo_www = models.ImageField(
        "Logo na stronę WWW",
        upload_to="logo",
        help_text="""Plik w formacie bitmapowym, np. JPEG lub PNG,
        w rozdzielczości maks. 100x100""",
        blank=True,
        null=True,
    )
    logo_svg = models.FileField(
        "Logo wektorowe (SVG)", upload_to="logo_svg", blank=True, null=True
    )
    favicon_ico = models.FileField(
        "Ikona ulubionych (favicon)", upload_to="favicon", blank=True, null=True
    )

    pbn_uid = models.ForeignKey(
        "pbn_api.Institution",
        verbose_name="Odpowiednik w PBN",
        on_delete=SET_NULL,
        null=True,
        blank=True,
    )

    obca_jednostka = models.ForeignKey(
        "bpp.Jednostka",
        SET_NULL,
        null=True,
        blank=True,
        help_text="""
    Jednostka skupiająca autorów nieindeksowanych, nie będących pracownikami uczelni. Procedury importujące
    dane z zewnętrznych systemów informatycznych będą przypisywać do tej jednostki osoby, które zakończyły
    pracę na uczelni. """,
        related_name="obca_jednostka",
    )

    ilosc_jednostek_na_strone = models.PositiveIntegerField(
        default=150,
        validators=[MinValueValidator(1), MaxValueValidator(10000)],
        help_text="""Ilość jednostek wyświetlanych na podstronie prezentacji
        danych dla użytkownika końcowego (strona główna -> przeglądaj -> jednostki)""",
    )

    pokazuj_tylko_jednostki_nadrzedne = models.BooleanField(
        default=False,
        help_text="""Pokazuj tylko jednostki nadrzędne na stronie prezentacji
        danych dla użytkownika końcowego""",
    )

    ranking_autorow_rozbij_domyslnie = models.BooleanField(
        'Zaznacz domyślnie "Rozbij punktację na jednostki i wydziały" dla rankingu autorów',
        default=False,
    )

    pokazuj_punktacje_wewnetrzna = models.BooleanField(
        "Pokazuj punktację wewnętrzną na stronie rekordu", default=True
    )
    pokazuj_index_copernicus = models.BooleanField(
        "Pokazuj Index Copernicus na stronie rekordu", default=True
    )
    pokazuj_punktacja_snip = models.BooleanField(
        "Pokazuj punktację SNIP na stronie rekordu", default=True
    )
    pokazuj_status_korekty = OpcjaWyswietlaniaField(
        "Pokazuj status korekty na stronie rekordu",
    )

    pokazuj_ranking_autorow = OpcjaWyswietlaniaField(
        "Pokazuj ranking autorów",
    )

    pokazuj_raport_uczelni = OpcjaWyswietlaniaField(
        "Pokazuj raport uczelni", default=OpcjaWyswietlaniaField.POKAZUJ_NIGDY
    )

    pokazuj_raport_autorow = OpcjaWyswietlaniaField("Pokazuj raport autorów")

    pokazuj_raport_jednostek = OpcjaWyswietlaniaField(
        "Pokazuj raport jednostek", default=OpcjaWyswietlaniaField.POKAZUJ_ZALOGOWANYM
    )

    pokazuj_raport_wydzialow = OpcjaWyswietlaniaField(
        "Pokazuj raport wydziałów", default=OpcjaWyswietlaniaField.POKAZUJ_ZALOGOWANYM
    )

    pokazuj_raport_dla_komisji_centralnej = OpcjaWyswietlaniaField(
        "Pokazuj raport dla Komisji Centralnej"
    )

    pokazuj_praca_recenzowana = OpcjaWyswietlaniaField(
        'Pokazuj opcję "Praca recenzowana"'
    )

    pokazuj_formularz_zglaszania_publikacji = OpcjaWyswietlaniaField(
        "Pokazuj opcję 'Zgłoś nową publikację'",
        help_text="Czy pokazywać formularz zgłaszania publikacji?",
    )

    domyslnie_afiliuje = models.BooleanField(
        "Domyślnie zaznaczaj, że autor afiliuje",
        help_text="""Przy powiązaniach autor + wydawnictwo, zaznaczaj domyślnie,
        że autor afiliuje do jednostki, która jest wpisywana.""",
        default=True,
    )

    pokazuj_liczbe_cytowan_w_rankingu = OpcjaWyswietlaniaField(
        "Pokazuj liczbę cytowań w rankingu"
    )

    pokazuj_liczbe_cytowan_na_stronie_autora = OpcjaWyswietlaniaField(
        "Pokazuj liczbę cytowań na podstronie autora",
        help_text="""Liczba cytowań będzie wyświetlana, gdy większa od 0""",
    )

    pokazuj_tabele_slotow_na_stronie_rekordu = OpcjaWyswietlaniaField(
        "Pokazuj tabelę slotów na stronie rekordu",
        default=OpcjaWyswietlaniaField.POKAZUJ_ZALOGOWANYM,
    )

    pokazuj_raport_slotow_autor = OpcjaWyswietlaniaField(
        "Pokazuj raport slotów - autor",
        default=OpcjaWyswietlaniaField.POKAZUJ_ZALOGOWANYM,
    )

    pokazuj_raport_slotow_zerowy = OpcjaWyswietlaniaField(
        "Pokazuj raport slotów zerowy",
        default=OpcjaWyswietlaniaField.POKAZUJ_ZALOGOWANYM,
    )

    pokazuj_raport_slotow_uczelnia = OpcjaWyswietlaniaField(
        "Pokazuj raport slotów - uczelnia",
        default=OpcjaWyswietlaniaField.POKAZUJ_ZALOGOWANYM,
    )

    wymagaj_informacji_o_oplatach = models.BooleanField(
        "Wymagaj informacji o opłatach",
        default=True,
        help_text="Gdy zaznaczone, moduł 'Zgłaszanie publikacji' będzie wyświetlać użytkownikowi formularz "
        "informacji o opłatach za publikację w przypadku zgłaszania artykułu lub monografii. "
        "Gdy odznaczone, taki formularz nie bedzie wyświetlany, niezależnie od rodzaju "
        "zgłaszanej publikacji. ",
    )

    wydruk_logo = models.BooleanField("Pokazuj logo na wydrukach", default=False)

    wydruk_logo_szerokosc = models.SmallIntegerField(
        "Szerokość logo na wydrukach",
        default=250,
        help_text="Podaj wartość w pikselach. Wysokość zostanie przeskalowana"
        " proporcjonalnie. ",
    )

    wydruk_parametry_zapytania = models.BooleanField(
        "Pokazuj parametry zapytania na wydrukach", default=True
    )

    wyszukiwanie_rekordy_na_strone_anonim = models.SmallIntegerField(
        "Ilość rekordów na stronę - anonim",
        default=200,
        help_text="Ilość rekordów w wyszukiwaniu powyżej której znika opcja"
        '"Pokaż wszystkie" i "Drukuj" dla użytkownika anonimowego. '
        "Nie jest zalecane ustawianie powyżej 500. ",
    )

    wyszukiwanie_rekordy_na_strone_zalogowany = models.SmallIntegerField(
        "Ilość rekordów na stronę - anonim",
        default=10000,
        help_text="Ilość rekordów w wyszukiwaniu powyżej której znika opcja"
        '"Pokaż wszystkie" i "Drukuj" dla użytkownika zalogowanego. '
        "Nie jest zalecane ustawianie powyżej 10000. ",
    )

    podpowiadaj_dyscypliny = models.BooleanField(
        default=True,
        help_text="""W sytuacji gdy to pole ma wartość "PRAWDA", system będzie podpowiadał dyscyplinę
        naukową dla powiązania rekordu wydawnictwa i autora w sytuacji, gdy autor ma na dany rok
        określoną tylko jedną dyscyplinę. W sytuacji przypisania dla autora dwóch dyscyplin na dany rok,
        pożądaną dyscyplinę będzie trzeba wybrać ręcznie, niezależnie od ustawienia tego pola. """,
    )

    sortuj_jednostki_alfabetycznie = models.BooleanField(
        default=True,
        help_text="""Jeżeli ustawione na 'FAŁSZ', sortowanie jednostek będzie odbywało się ręcznie
        tzn za pomocą ustalonej przez administratora systemu kolejności. """,
    )

    clarivate_username = models.CharField(
        verbose_name="Nazwa użytkownika", null=True, blank=True, max_length=50
    )

    clarivate_password = models.CharField(
        verbose_name="Hasło", null=True, blank=True, max_length=50
    )

    DO_ROKU = Choices(
        (
            const.DO_STYCZNIA_POPRZEDNI_POTEM_OBECNY,
            "do stycznia poprzedni, potem obecny",
        ),
        (const.NAJWIEKSZY_REKORD, "najwiekszy rok rekordu w bazie"),
    )

    metoda_do_roku_formularze = models.CharField(
        "Data w polu 'do roku' w formularzach",
        choices=DO_ROKU,
        default=const.DO_STYCZNIA_POPRZEDNI_POTEM_OBECNY,
        max_length=30,
        help_text="Decyduje o sposobie wyświetlania maksymalnej daty 'Do roku' w formularzach. ",
    )

    pbn_integracja = models.BooleanField(
        default=False,
        verbose_name="Czy używać integracji z PBN?",
        help_text="Wymaga prawidłowo wypełnionych pól 'Adres API w PBN', 'Nazwa aplikacji w PBN', "
        "'Token aplikacji w PBN' oraz łączności między serwerem BPP a serwerem PBN. ",
    )

    pbn_aktualizuj_na_biezaco = models.BooleanField(
        default=False,
        verbose_name="Włącz opcjonalną aktualizację przy edycji",
        help_text="""Aktualizuj rekordy w PBN przy zapisie rekordu, gdy redaktor kliknie odpowiedni przycisk.
        Wybranie tej opcji spowoduje, ze na podstronach modułu redagowania dla wydawnictw pojawią się
        przyciski 'Zapisz i wyślij do PBN'. """,
    )

    pbn_api_root = models.URLField(
        "Adres API w PBN", default="https://pbn-micro-alpha.opi.org.pl"
    )
    pbn_app_name = models.CharField(
        "Nazwa aplikacji w PBN", blank=True, null=True, max_length=128
    )
    pbn_app_token = models.CharField(
        "Token aplikacji w PBN", blank=True, null=True, max_length=128
    )
    pbn_api_kasuj_przed_wysylka = models.BooleanField(
        "Kasuj oświadczenia rekordu przed wysłaniem do PBN", default=False
    )
    pbn_api_nie_wysylaj_prac_bez_pk = models.BooleanField(
        "Nie wysyłaj do PBN prac z PK=0", default=False
    )

    pbn_api_afiliacja_zawsze_na_uczelnie = models.BooleanField(
        "Wysyłaj zawsze PBN UID uczelni jako afiliację",
        default=False,
        help_text="Jeżeli praca jest w jednostce z wypełnionym PBN UID bądź w jednostce "
        "innej-niż-obca, zatrudniającej-pracowników, to zaznaczenie tej opcji spowoduje, ze  zamiast PBN "
        "UID tej jednostki zostanie użyty PBN UID uczelni, co efektywnie "
        "spowoduje, że afiliacje w PBN będą na uczelnię, nie zaś na konkretną jednostkę. ",
    )

    pbn_api_user = models.ForeignKey(
        "bpp.BppUser",
        on_delete=SET_NULL,
        verbose_name="Użytkownik BPP dla PBN API",
        help_text="Użytkownik po stronie BPP który bedzie odpowiedzialny za operacje "
        "przeprowadzane w tle przez procesy działające na serwerze i pobierające dane z PBN np w nocy. Jeżeli ten "
        "użytkownik dokona autoryzacji w PBN za pomocą przeglądarki, to możliwe będzie również aktualizowanie "
        "(wgrywanie) rekordów przez niego na serwer PBN. ",
        blank=True,
        null=True,
    )

    objects = UczelniaManager()

    class Meta:
        verbose_name = "uczelnia"
        verbose_name_plural = "uczelnie"
        app_label = "bpp"

    def get_absolute_url(self):
        return reverse("bpp:browse_uczelnia", args=(self.slug,))

    def wydzialy(self):
        """Widoczne wydziały -- do pokazania na WWW"""
        from .wydzial import Wydzial

        return Wydzial.objects.filter(uczelnia=self, widoczny=True)

    def clean(self):
        if self.obca_jednostka is not None:
            if self.obca_jednostka.skupia_pracownikow:
                raise ValidationError(
                    {
                        "obca_jednostka": "Obca jednostka musi faktycznie być obca. Wybrana ma ustaloną wartość "
                        "'skupia pracowników' na PRAWDA, czyli nie jest obcą jednostką. "
                    }
                )

        if self.pbn_aktualizuj_na_biezaco and not self.pbn_integracja:
            raise ValidationError(
                {
                    "pbn_aktualizuj_na_biezaco": "Jeżeli nie używasz integracji z PBN, odznacz również i to pole. "
                }
            )

        if self.pbn_integracja:
            # Wykonaj próbne pobranie rekordu z PBNu
            client = self.pbn_client()
            try:
                client.get_languages()
            except Exception as e:
                raise ValidationError(
                    {
                        "pbn_integracja": f"Nie można pobrać przykładowych rekordów przez API celem weryfikacji. "
                        f"Kod błędu: {e}. "
                        f"Jeżeli bład nie dotyczy problemów z siecią lub z autoryzacją, skontaktuj się"
                        f"z administratorem. "
                    }
                )

    def save(self, *args, **kw):
        self.clean()
        return super().save(*args, **kw)

    def wosclient(self):
        """
        :rtype: wosclient.wosclient.WoSClient
        """
        if not self.clarivate_username:
            raise ImproperlyConfigured("Brak użytkownika API w konfiguracji serwera")

        if not self.clarivate_password:
            raise ImproperlyConfigured("Brak hasła do API w konfiguracji serwera")

        from wosclient.wosclient import WoSClient

        return WoSClient(self.clarivate_username, self.clarivate_password)

    def pbn_client(self, pbn_user_token=None) -> "client.PBNClient":
        """
        Zwraca klienta PBNu
        """
        from pbn_api import client

        class UczelniaTransport(client.RequestsTransport):
            def authorize(self, base_url, app_id, token):
                if not pbn_user_token:
                    raise WillNotExportError(
                        "Najpierw wykonaj autoryzację w PBN API za pomocą menu dostępnego "
                        "na głównej stronie (operacje -> autoryzuj w PBN)"
                    )
                self.access_token = pbn_user_token
                return True

        if not self.pbn_app_name:
            raise ImproperlyConfigured("Brak nazwy aplikacji dla API PBN")

        if not self.pbn_app_token:
            raise ImproperlyConfigured("Brak tokena aplikacji dla API PBN")

        transport = UczelniaTransport(
            self.pbn_app_name, self.pbn_app_token, self.pbn_api_root, pbn_user_token
        )
        return client.PBNClient(transport)

    def ukryte_statusy(self, dla_funkcji: str) -> List[int]:
        """
        :param dla_funkcji: "sloty", "raporty", "multiwyszukiwarka", "rankingi"
        :return: lista numerów PK obiektów :class:`bpp.models.system.Status_Korekty`
        """
        return self.ukryj_status_korekty_set.filter(**{dla_funkcji: True}).values_list(
            "status_korekty", flat=True
        )

    def sprawdz_uprawnienie(self, attr, request, ignoruj_grupe=None):
        res = getattr(self, f"pokazuj_{attr}")
        if res == OpcjaWyswietlaniaField.POKAZUJ_ZAWSZE:
            return True

        if res == OpcjaWyswietlaniaField.POKAZUJ_ZALOGOWANYM:
            if request.user.is_anonymous:
                return False

            if request.user.is_superuser:
                return True

            if str(ignoruj_grupe) != "ignoruj_grupe":
                if request.user.groups.filter(name=GR_RAPORTY_WYSWIETLANIE).exists():
                    # Pokazuj zalogowanym ale wyłącznie gdy są w grupei GR_RAPORTY_WYSWIETLANIE
                    # chyba, ze jest podany parametr ignoruj_grupe
                    return True
                else:
                    return False

            return True

        if res == OpcjaWyswietlaniaField.POKAZUJ_GDY_W_ZESPOLE:
            if request.user.is_anonymous:
                return False
            if not request.user.is_staff:
                return False
            return True

        if res == OpcjaWyswietlaniaField.POKAZUJ_NIGDY:
            return False

        raise NotImplementedError()

    def autorzy_z_uczelni(self):
        # Zwróc listę wszystkich autorów, którzy są przypisani do
        # nie-obcych jednostek
        from bpp.models.struktura import Autor

        return Autor.objects.filter(
            autor_jednostka__jednostka__skupia_pracownikow=True,
            autor_jednostka__jednostka__uczelnia=self,
            autor_jednostka__jednostka__pk__gte=0,
        ).distinct()


class Ukryj_Status_Korekty(models.Model):
    uczelnia = models.ForeignKey(Uczelnia, on_delete=models.CASCADE)
    status_korekty = models.ForeignKey("Status_Korekty", on_delete=models.CASCADE)

    multiwyszukiwarka = models.BooleanField(
        default=True,
        help_text="Nie dotyczy użytkownika zalogowanego. Użytkownik zalogowany widzi wszystkie prace "
        "w wyszukiwaniu. ",
    )
    podglad = models.BooleanField(
        default=True,
        help_text="Uniemożliwia podgląd prac na stronie szczegółów rekordu oraz uniemożliwia wyszukanie takich "
        "prac przez pole globalnego wyszukania.",
    )
    raporty = models.BooleanField(
        "Raporty",
        default=True,
        help_text="Ukrywa prace w raporcie autora, " "jednostki, uczelni",
    )
    rankingi = models.BooleanField("Rankingi", default=True)
    sloty = models.BooleanField(
        "Raporty slotów",
        default=True,
        help_text="Prace o wybranym statusie nie będą miały liczonych punktów i slotów w chwili"
        "zapisywania rekordu do bazy danych. Jeżeli zmieniasz to ustawienie dla prac które już są w bazie danych "
        "to ich punktacja zniknie z bazy w dniu następnym (skasowana zostanie podczas nocnego przeindeksowania bazy).",
    )
    api = models.BooleanField(
        "API",
        default=True,
        help_text="Dotyczy ukrywania prac w API JSON-REST oraz OAI-PMH",
    )

    def __str__(self):
        res = (
            f'ukryj "{self.status_korekty}" dla '
            f"{'multiwyszukiwarki, ' if self.multiwyszukiwarka else ''}"
            f"{'raportów, ' if self.raporty else ''}"
            f"{'rankingów, ' if self.rankingi else ''}"
            f"{'slotów. ' if self.sloty else ''}"
        )

        if res.endswith(", "):
            res = res[:-2] + ". "
        return res

    class Meta:
        unique_together = [("uczelnia", "status_korekty")]
        verbose_name = "ustawienie ukrywania statusu korekty"
        verbose_name_plural = "ustawienia ukrywania statusów korekt"
